package com.workflow.database;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import java.net.URISyntaxException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;


import org.apache.log4j.Logger;
import org.hibernate.cfg.Configuration;
import org.hibernate.cfg.Environment;

public abstract class HibernateConfiguration {
	private static final Logger logger = Logger.getLogger(HibernateConfiguration.class);
	private static Configuration hibernateConfiguration;

	private static Connection connection;

	
	public static Configuration getInstance() {
		return hibernateConfiguration;
	}

	public static void initialize(String username, String password, String hostname, String schemaname, String showSQL, String path, String db) {
		hibernateConfiguration = new Configuration();

		// add mappings
		hibernateConfiguration.addResource("com/workflow/database/beans/Sample.hbm.xml");
		hibernateConfiguration.addResource("com/workflow/database/beans/Study.hbm.xml");
		hibernateConfiguration.addResource("com/workflow/database/beans/User.hbm.xml");
		hibernateConfiguration.addResource("com/workflow/database/beans/Snp.hbm.xml");
		hibernateConfiguration.addResource("com/workflow/database/beans/Hwe.hbm.xml");
		hibernateConfiguration.addResource("com/workflow/database/beans/Discordance.hbm.xml");
		hibernateConfiguration.addResource("com/workflow/database/beans/Sexsnp.hbm.xml");
		hibernateConfiguration.addResource("com/workflow/database/beans/Genotype.hbm.xml");
		hibernateConfiguration.addResource("com/workflow/database/beans/Hapmap.hbm.xml");

		// set properties
		// connection settings
		boolean mysql =true;
		if (db.toLowerCase().equals("mysql"))
			{mysql=true;}
	
		else{ mysql=false;}
		
		if (mysql){
		hibernateConfiguration.setProperty(Environment.DRIVER,	"org.gjt.mm.mysql.Driver");
		hibernateConfiguration.setProperty(Environment.DIALECT,	"org.hibernate.dialect.MySQL5Dialect");
		hibernateConfiguration.setProperty(Environment.URL, createConnectionString(hostname, schemaname));
		}
		else
		{
			hibernateConfiguration.setProperty(Environment.DRIVER,	"org.h2.Driver");
			hibernateConfiguration.setProperty(Environment.DIALECT,	"org.hibernate.dialect.H2Dialect");
			hibernateConfiguration.setProperty(Environment.URL, createConnectionStringH2(path+File.separator+schemaname));
			try {
				Class.forName("org.h2.Driver");
			} catch (ClassNotFoundException e1) {
				e1.printStackTrace();
			}
			File file = new File(path+File.separator+schemaname+".h2.db");
			boolean exists = file.exists();
			 try {
				connection = DriverManager.getConnection("jdbc:h2:" + path+File.separator+schemaname, username, password);
			} catch (SQLException e1) {
				e1.printStackTrace();
			}
			
			if (!exists) 
			{
				logger.debug("Creating tables...");
				
				
				try {
					executeSQL(path+"\\snpflow.sql");
					connection.commit();
				} catch (SQLException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				} catch (URISyntaxException e) {
					e.printStackTrace();
				}
			}
		}
		
		hibernateConfiguration.setProperty(Environment.AUTOCOMMIT,	"true");
		hibernateConfiguration.setProperty(Environment.USER, username);
		hibernateConfiguration.setProperty(Environment.PASS, password);
		hibernateConfiguration.setProperty(Environment.C3P0_ACQUIRE_INCREMENT, "1");
		hibernateConfiguration.setProperty(Environment.C3P0_TIMEOUT, "120");
		hibernateConfiguration.setProperty(Environment.C3P0_IDLE_TEST_PERIOD, "60");
		hibernateConfiguration.setProperty(Environment.C3P0_ACQUIRE_INCREMENT, "1");
		hibernateConfiguration.setProperty(Environment.C3P0_MAX_SIZE, "10");
		hibernateConfiguration.setProperty(Environment.C3P0_MIN_SIZE, "1");
		hibernateConfiguration.setProperty(Environment.C3P0_MAX_STATEMENTS, "10");
		
		hibernateConfiguration.setProperty("c3p0.preferredTestQuery", "SELECT 1");
		hibernateConfiguration.setProperty("c3p0.testConnectionOnCheckout", "true");

		hibernateConfiguration.setProperty(Environment.CURRENT_SESSION_CONTEXT_CLASS, "thread");

		hibernateConfiguration.setProperty(Environment.SHOW_SQL, showSQL);
		hibernateConfiguration.setProperty("transaction.factory_class", "org.hibernate.transaction.JDBCTransactionFactory");
	}

	private static String createConnectionString(String hostname, String schemaname){
		return "jdbc:mysql://"+hostname+"/"+schemaname+"?useJvmCharsetConverters=true&autoReconnect=true";
	}

	private static String createConnectionStringH2(String schemaname){
		return "jdbc:h2:"+schemaname;
	}
	public static boolean testDBConnection(){
		try {
			WorkflowDAO.getCurrentSession().createSQLQuery("SELECT 1 FROM dual")
					.uniqueResult();
			return true;
		} catch (Exception e) {
			logger.error("Error in testDBConnection");
			e.printStackTrace();
			return false;
		}
	}

	  static String getFileResource(String resourceName) throws IOException
	  { StringBuffer content = new StringBuffer();
	    String line;
	    InputStream rs = Class.class.getResourceAsStream(resourceName);
	    BufferedReader in = new BufferedReader(new InputStreamReader(rs, "UTF-8"));
	    while((line = in.readLine()) != null) content.append(line).append("\n");
	    in.close();
	    return content.toString();
	  }

	public static void executeSQL(String filename) throws SQLException,
			IOException, URISyntaxException {
		String sqlContent = readFileAsString(filename);

		PreparedStatement ps = connection.prepareStatement(sqlContent);
		ps.executeUpdate();
	}

	public static String readFileAsString(String filename) throws java.io.IOException, URISyntaxException {

		InputStream is = new FileInputStream(filename);
		DataInputStream in = new DataInputStream(is);
		BufferedReader br = new BufferedReader(new InputStreamReader(in));
		String strLine;
		StringBuilder builder = new StringBuilder();
		while ((strLine = br.readLine()) != null) {
			builder.append("\n");
			builder.append(strLine);
		}
		in.close();
		System.out.println("BUILDER " + builder.toString());
		return builder.toString();
	}

}
