package com.workflow.database;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;

public class WorkflowDAO {
	final private static SessionFactory sessionFactory;
	final private static ThreadLocal<Session> threadSession = new ThreadLocal<Session>();
	final private static ThreadLocal<Transaction> threadTransaction = new ThreadLocal<Transaction>();

	static {
		sessionFactory = HibernateConfiguration.getInstance().buildSessionFactory();
	}

	public static Session getCurrentSession() {
		Session session = (Session) threadSession.get();
		if (session == null || !session.isOpen()) {

			session = sessionFactory.openSession();
			threadSession.set(session);
		}

		return session;
	}
	
	public static Transaction getCurrentTransaction() {
		Transaction transaction = (Transaction) threadTransaction.get();
		if (transaction == null || !transaction.isActive()) {
			transaction = getCurrentSession().beginTransaction();

			threadTransaction.set(transaction);
		}

		return transaction;
	}
	
	public static boolean transactionExists(){
		return threadTransaction.get() != null;
	}

	public static void closeSession() {
		Session session = (Session) threadSession.get();
		threadSession.set(null);

		if (session != null && session.isOpen()) {
			session.close();
		}
	}

	public static Session beginTransaction() {
		Session session = getCurrentSession();

		Transaction tx = (Transaction) threadTransaction.get();
		if (tx == null) {
			tx = getCurrentSession().beginTransaction();
			threadTransaction.set(tx);
		}

		return session;
	}

	public static void commitTransaction() {
		Transaction tx = (Transaction) threadTransaction.get();
		if (tx != null && !tx.wasCommitted() && !tx.wasRolledBack()) {
			Session session = (Session) threadSession.get();
			if (session != null && session.isOpen()) {
				tx.commit();
			}

			threadTransaction.set(null);
		}
	}

	public static void rollBackTransaction() {
		Transaction tx = (Transaction) threadTransaction.get();
		if (tx != null && !tx.wasCommitted() && !tx.wasRolledBack()) {
			tx.rollback();
			threadTransaction.set(null);
		}
		closeSession();
	}
}
