package com.workflow.management;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import javax.sound.sampled.DataLine;

import org.apache.log4j.Logger;
import org.hibernate.criterion.Example;

import com.workflow.management.beans.ExampleHolder;
import com.workflow.management.beans.SexSnpHolder;

public class DataImporter {
	private static final Logger logger = Logger.getLogger(DataImporter.class);
	private static final String ZIP = "application/zip";
	private static final String ZIP_OCTET = "application/octet-stream";
	private static final String XZIP = "application/x-zip-compressed";
	private static final String TEXT = "text/plain";

	/**
	 * 
	 * @param in
	 * @param contentType
	 * @param method
	 *            1=Taqman, 2=Sequenom, 3=other
	 * @return
	 * @throws IOException
	 */
	public static ConcurrentHashMap<String, List<ExampleHolder>> processFile(
			InputStream in, String contentType, String method, String fileName)
			throws Exception {
		if (contentType.equalsIgnoreCase(TEXT)) {
			if (method.toString().equals("1")) {
				ConcurrentHashMap<String, List<ExampleHolder>> dataMap = new ConcurrentHashMap<String, List<ExampleHolder>>();
				dataMap.put("key", parseTxtFileTaqman(in));
				return dataMap;
			} else if (method.toString().equals("2")) {
				ConcurrentHashMap<String, List<ExampleHolder>> dataMap = new ConcurrentHashMap<String, List<ExampleHolder>>();
				parseTxtFileSequenom(in, dataMap, fileName);
				System.out.println("length dataMap " + dataMap.size());
				return dataMap;
			} else
				return null;
		} else if (contentType.equalsIgnoreCase(ZIP)
				|| contentType.equalsIgnoreCase(ZIP_OCTET)
				|| contentType.equalsIgnoreCase(XZIP)) {
			return parseZipFile(in, method);
		} else {
			throw new IllegalArgumentException("Not a valid data file given");
		}
	}

	public static ConcurrentHashMap<String, List<SexSnpHolder>> processFileSexSnp(
			InputStream in, String contentType, String fileName)
			throws IOException {
		if (contentType.equalsIgnoreCase(TEXT)) {

			ConcurrentHashMap<String, List<SexSnpHolder>> dataMap = new ConcurrentHashMap<String, List<SexSnpHolder>>();

			dataMap.put("key", parseTxtFileSexSnp(in));
			return dataMap;
		} else
			return null;
	}

	/**
	 * TAQMAN PARSER
	 * 
	 * @param in
	 * @return
	 */
	private static List<ExampleHolder> parseTxtFileTaqman(InputStream in) {
		InputStreamReader ir = new InputStreamReader(in);
		BufferedReader br = new BufferedReader(ir);
		List<ExampleHolder> dataList = new LinkedList<ExampleHolder>();

		int lineCount = 0;
		String line = "";
		String fileName = "";
		try {
			while ((line = br.readLine()) != null) {
				if (lineCount == 1) {
					// parseFileName
					fileName = line.split("\t")[1];
				} else if (lineCount > 11 && line != null && line.length() > 0) {
					ExampleHolder eh = new ExampleHolder();
					eh.setFileName(fileName);
					String lineContent[] = line.split("\t");
					eh.setWell(lineContent[0]);
					eh.setSampleName(lineContent[1]);
					eh.setCall(lineContent[5]);
					dataList.add(eh);
				}
				++lineCount;
			}
		} catch (IOException e) {

			e.printStackTrace();
		}
		return dataList;
	}

	/**
	 * SEQUENOM PARSER
	 * 
	 * @param in
	 * @param snp
	 * @return
	 * @throws Exception
	 */
	private static ConcurrentHashMap<String, List<ExampleHolder>> parseTxtFileSequenom(
			InputStream in,
			ConcurrentHashMap<String, List<ExampleHolder>> dataMap,
			String fileName) throws Exception {
		logger.debug("sequenom");
		InputStreamReader ir = new InputStreamReader(in);
		BufferedReader br = new BufferedReader(ir);
		// HashMap<String, List<ExampleHolder>> dataMap = new HashMap<String,
		// List<ExampleHolder>>();

		int lineCount = 0;
		String line = "";
		String snp = "";
		String rsSnp = "";
		String call = "";
		String sampleId = "";
		String well = "";

		List<ExampleHolder> dataList = null;
		while ((line = br.readLine()) != null) {

			if (lineCount > 1) { // ignore first 2line

				if (line.trim().length() > 0) {
					String lineContent[] = line.split("\t");
					sampleId = lineContent[0];
					call = lineContent[2];
					snp = lineContent[3];
					rsSnp = "rs" + snp.replace("rs", "");
					well = lineContent[4];

					ExampleHolder eh = new ExampleHolder();

					eh.setFileName(fileName);
					eh.setSampleName(sampleId);

					if (call.trim().length() == 1)
						eh.setCall("rs" + snp.replace("rs", "") + "_" + call); // CALL:
																// rs01234567_C
					else if (call.trim().length() == 2)
						eh.setCall("Both_rs" + snp.replace("rs", "") + "_" + call); // CALL:
																	// Both_rs01234567_CT
					else
						eh.setCall("Undetermined_rs" + snp.replace("rs", "")); // CALL:
																// Undetermined_rs01234567
					eh.setWell(well);

					dataList = null;
					dataList = dataMap.get(rsSnp);
					if (dataList == null) {
						dataList = new LinkedList<ExampleHolder>();
						dataMap.put(rsSnp, dataList);
					}
					dataList.add(eh);
				}
			} else
				++lineCount;
		}
		logger.debug("DATAMAP.SIZE " + dataMap.size() + " " + lineCount);

		return dataMap;
	}

	/**
	 * SexSNP PARSER
	 * 
	 * @param in
	 * @return
	 */
	private static List<SexSnpHolder> parseTxtFileSexSnp(InputStream in) {
		InputStreamReader ir = new InputStreamReader(in);
		BufferedReader br = new BufferedReader(ir);
		List<SexSnpHolder> dataList = new LinkedList<SexSnpHolder>();

		String line = "";

		try {
			System.out.println(br.readLine()); // skip header
			while ((line = br.readLine()) != null) {

				SexSnpHolder sh = new SexSnpHolder();
				String lineContent[] = line.split("\t");

				sh.setSampleName(lineContent[0]);
				sh.setPhenotype(lineContent[1]);
				dataList.add(sh);
			}
		} catch (IOException e) {

			e.printStackTrace();
		}

		return dataList;
	}

	private static ConcurrentHashMap<String, List<ExampleHolder>> parseZipFile(			InputStream in, String method) throws Exception {
		List<ExampleHolder> dataList = new LinkedList<ExampleHolder>();
		ZipInputStream zipin = new ZipInputStream(in);
		ZipEntry zEntry;
		ConcurrentHashMap<String, List<ExampleHolder>> dataMap = new ConcurrentHashMap<String, List<ExampleHolder>>();
		System.out.println();
		zEntry = zipin.getNextEntry();
	

		while (zEntry != null) {
			if (method.equals("1")) {
				if (dataMap.get("key") == null)
					dataMap.put("key", parseTxtFileTaqman(zipin));
				else
					dataMap.get("key").addAll(parseTxtFileTaqman(zipin));
			}

			if (method.equals("2")) {
				dataMap = parseTxtFileSequenom(zipin, dataMap, zEntry.getName());
			}
			zipin.closeEntry();
			zEntry = zipin.getNextEntry();
		}
		zipin.close();

		return dataMap;
	}

}