package com.workflow.management;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLEncoder;
import java.text.DecimalFormat;
import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.regex.Pattern;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import com.sun.org.apache.bcel.internal.generic.GETSTATIC;
import com.workflow.database.DatabaseUtils;
import com.workflow.database.beans.Hapmap;
import com.workflow.database.beans.Hwe;
import com.workflow.database.beans.Snp;
import com.workflow.management.util.CalcUtils;

public class HapMapManagement {


	

	public static void writeHapMap(Snp snp) throws Exception {
		Set<Hapmap> hapmaps = new HashSet<Hapmap>();

		final String query = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
			+ "<!DOCTYPE Query>"
			+ "<Query  virtualSchemaName = \"rel27_NCBI_Build36\" formatter = \"TSV\" header = \"0\" uniqueRows = \"0\" count = \"\" datasetConfigVersion = \"0.6\" >"
			+ "	<Dataset name = \"hm27_variation\" interface = \"default\" >"
			+ "		<Filter name = \"marker_name\" value = \"" + snp.getHapmapLookup() + "\"/>"
			+ "		<Attribute name = \"otherhom_gt\" />"//0
			+ "		<Attribute name = \"refhom_gt\" />"//1
			+ "		<Attribute name = \"het_gt\" />"//2
			+ "		<Attribute name = \"otherhom_gtfreq\" />"//3
			+ " 	<Attribute name = \"refhom_gtfreq\" />"//4
			+ "		<Attribute name = \"het_gtfreq\" />"//5
			+ "		<Attribute name = \"otherhom_gtcount\" />"//6
			+ "		<Attribute name = \"refhom_gtcount\" />"//7
			+ "		<Attribute name = \"het_gtcount\" />"//8
			+ "		<Attribute name = \"total_gtcount\" />"//9
			+ "		<Attribute name = \"pop_code_genotype\" />" //10
			+ "	</Dataset>"
			+ "</Query>";

		final String encodedQuery = URLEncoder.encode(query, "UTF-8");

		final URL url = new URL(
			"http://hapmap.ncbi.nlm.nih.gov/biomart/martservice?query="
			+ encodedQuery);

		BufferedReader in = new BufferedReader(new InputStreamReader(url.openStream()));
		String inputLine;

		while ((inputLine = in.readLine()) != null) {
			
			System.out.println(inputLine);
			String[] line = inputLine.split("\t");
			Hapmap hm = new Hapmap();

			String nameHomo1 = line[0].replace("/", "");

			double homo1 = 0.0;
			double homoFreq1 = 0.0;
			double homo2 = 0.0;
			double homoFreq2 = 0.0;

			if (nameHomo1.equals(snp.getNameHomo1())) {
				homo1 = Double.parseDouble(line[6]);
				homoFreq1 = Double.parseDouble(line[3]);
				homo2 = Double.parseDouble(line[7]);
				homoFreq2 = Double.parseDouble(line[4]);
			} else if (nameHomo1.equals(snp.getNameHomo2())) {
				homo1 = Double.parseDouble(line[7]);
				homoFreq1 = Double.parseDouble(line[4]);
				homo2 = Double.parseDouble(line[6]);
				homoFreq2 = Double.parseDouble(line[3]);
			} else {
				if ((nameHomo1.equals("AA") && snp.getNameHomo1().equals("TT")) ||
					(nameHomo1.equals("CC") && snp.getNameHomo1().equals("GG"))){
					homo1 = Double.parseDouble(line[6]);
					homoFreq1 = Double.parseDouble(line[3]);
					homo2 = Double.parseDouble(line[7]);
					homoFreq2 = Double.parseDouble(line[4]);
				} else if ((nameHomo1.equals("AA") && snp.getNameHomo2().equals("TT")) ||
							(nameHomo1.equals("CC") && snp.getNameHomo2().equals("GG"))) {
					homo1 = Double.parseDouble(line[7]);
					homoFreq1 = Double.parseDouble(line[4]);
					homo2 = Double.parseDouble(line[6]);
					homoFreq2 = Double.parseDouble(line[3]);
				} else if ((nameHomo1.equals("TT") && snp.getNameHomo1().equals("AA")) ||
					(nameHomo1.equals("GG") && snp.getNameHomo1().equals("CC"))) {
					homo1 = Double.parseDouble(line[6]);
					homoFreq1 = Double.parseDouble(line[3]);
					homo2 = Double.parseDouble(line[7]);
					homoFreq2 = Double.parseDouble(line[4]);
				} else if ((nameHomo1.equals("TT") && snp.getNameHomo1().equals("AA")) ||
					(nameHomo1.equals("GG") && snp.getNameHomo1().equals("CC"))) {
					homo1 = Double.parseDouble(line[7]);
					homoFreq1 = Double.parseDouble(line[4]);
					homo2 = Double.parseDouble(line[6]);
					homoFreq2 = Double.parseDouble(line[3]);
				} else {
					System.out.println("snp " + inputLine );
					return;
				}
			}

			double heteroFreq = Double.parseDouble(line[5]);
			double hetero = Double.parseDouble(line[8]);
			int total = Integer.parseInt(line[9]);
			String population = line[10];

			hm.setPopulation("HapMap "+population);
			hm.setTotal(total);
			double chi = calcHapmapChi(snp, homo1, hetero, homo2);
			hm.setChi(chi);
			hm.setHetero(hetero);
			hm.setHeteroFreq(heteroFreq);
			hm.setHomo1(homo1);
			hm.setHomoFreq1(homoFreq1);
			hm.setHomo2(homo2);
			hm.setHomoFreq2(homoFreq2);
			hm.setSnp(snp);

			hapmaps.add(hm);
		}
		in.close(); // end read HapMap
		DatabaseUtils.add(hapmaps);
		/////////////////////////////////////////////////append 1000Genomes Data
	
	}



	private static double calcHapmapChi(Snp snp, double homo1, double hetero, double homo2) {
		final Hwe hwe = snp.getHwe();

		double sum1 = ((double)hwe.getHomoObserved1() + (double)hwe.getHeteroObserved() + (double)hwe.getHomoObserved2());
		double sum2 = (double)hwe.getHomoObserved1() + homo1;
		double sum3 = homo1 + hetero + homo2 + (double)hwe.getHomoObserved1() + (double)hwe.getHeteroObserved() + (double)hwe.getHomoObserved2();
		double sum4 = (double)hwe.getHeteroObserved() + hetero;
		double sum5 = (double)hwe.getHomoObserved2() + homo2;
		double sum6 = homo1 + hetero + homo2;

		double ownExHo1 = (sum1 * sum2) / sum3;
		double ownExHet = (sum1 * sum4) / sum3;
		double ownExHo2 = (sum1 * sum5) / sum3;
		double hapExHo1 = (sum6 * sum2) / sum3;
		double hapExHet = (sum6 * sum4) / sum3;
		double hapExHo2 = (sum6 * sum5) / sum3;

		//((B13-G13)^2)/G13+((C13-H13)^2)/H13+((D13-I13)^2)/I13+((B14-G14)^2)/G14+((C14-H14)^2)/H14+((D14-I14)^2)/I14
		double chi =((Math.pow(homo1-hapExHo1, 2))/hapExHo1)+((Math.pow(hetero-hapExHet, 2))/hapExHet)+((Math.pow(homo2-hapExHo2, 2))/hapExHo2)+((Math.pow(hwe.getHomoObserved1()-ownExHo1, 2))/ownExHo1)+((Math.pow(hwe.getHeteroObserved()-ownExHet, 2))/ownExHet)+((Math.pow(hwe.getHomoObserved2()-ownExHo2, 2))/ownExHo2);
		double p = CalcUtils.calc_x_df(chi, 2);

		return Double.isNaN(p) ? 0.0 : p;
	}
	
	
}