package com.workflow.management;

import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import com.csvreader.CsvReader;

public class ProbeImporter {
	private static final String SAMPLE_ID = "sample_id";

	public static List<String> parseSampleIds(InputStream in, String contentType)
			throws IOException {
		try {
			return parseTxtFile(in);
		} catch (Exception e) {
			throw new IllegalArgumentException(
					"Not a valid data file given - column [" + SAMPLE_ID
							+ "] not found");
		}
	}

	private static List<String> parseTxtFile(InputStream in) throws IOException {
		CsvReader reader = new CsvReader(in, ';', Charset.defaultCharset());
		List<String> samples = new LinkedList<String>();

		reader.readHeaders();
		while (reader.readRecord()) {
			String sampleId = reader.get(SAMPLE_ID);
			if (sampleId.isEmpty())
				throw new IllegalArgumentException();
			samples.add(sampleId);
		}
		return samples;
	}

	public static HashMap<String, String[]> parseSamplesNanodrop(InputStream in)
			throws IOException {
		CsvReader reader = new CsvReader(in, ';', Charset.defaultCharset());
		HashMap<String, String[]> values = new HashMap<String, String[]>();

		reader.readHeaders();
		while (reader.readRecord()) {
			String tmpVal[] = new String[2];
			String sampleId = reader.get(0);
			String concentration = reader.get(4);
			String qualityValue = reader.get(7);

			tmpVal[0] = concentration;
			tmpVal[1] = qualityValue;

			values.put(sampleId, tmpVal);
		}

		return values;
	}

	public static HashMap<String, String[]> parseSamplesNanoquand(InputStream in)
			throws IOException {
		CsvReader reader = new CsvReader(in, '\t', Charset.defaultCharset());
		HashMap<String, String[]> values = new HashMap<String, String[]>();

		reader.readHeaders();
		while (reader.readRecord()) {
			String tmpVal[] = new String[2];
			String sampleId = reader.get(1);
			String concentration = reader.get(3);
			String qualityValue = reader.get(4);

			tmpVal[0] = concentration;
			tmpVal[1] = qualityValue;

			values.put(sampleId, tmpVal);
		}

		return values;
	}
}