package com.workflow.management;

import java.awt.Color;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import jxl.Cell;
import jxl.CellView;
import jxl.Range;
import jxl.Workbook;
import jxl.format.Alignment;
import jxl.format.CellFormat;
import jxl.format.Colour;
import jxl.write.DateFormat;
import jxl.write.DateTime;
import jxl.write.Label;
import jxl.write.NumberFormat;
import jxl.write.NumberFormats;
import jxl.write.WritableCellFormat;
import jxl.write.WritableSheet;
import jxl.write.WritableWorkbook;
import jxl.write.WriteException;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.math.stat.inference.TestUtils;
import org.apache.log4j.Logger;
import org.hibernate.Session;

import com.workflow.database.DatabaseUtils;
import com.workflow.database.WorkflowDAO;
import com.workflow.database.beans.Discordance;
import com.workflow.database.beans.Genotype;
import com.workflow.database.beans.Hapmap;
import com.workflow.database.beans.Hwe;
import com.workflow.database.beans.Sample;
import com.workflow.database.beans.Snp;
import com.workflow.database.beans.SnpContainer;
import com.workflow.database.beans.Study;
import com.workflow.management.beans.ExampleHolder;
import com.workflow.management.util.CalcUtils;
import com.workflow.servlet.InitServlet;

public class SNPManagement {
	private static final Logger logger = Logger.getLogger(SNPManagement.class);
	
	private static final char CARRIAGE = '\n';
	private static final char TAB = '\t';
	private static final String ER = "ER";
	private static final String NTC = "NTC";
	private static final String BOTH = "Both";
	private static final String UNDETERMINED = "Undetermined";
    private static String arrayHapmap[] = {"ASW", "CEU", "CHB", "CHD", "GIH", "JPT", "LWK", "MEX", "MKK", "TSI", "YRI", "", "AFR", "AMR", "ASN", "EUR"};
	
	public static void deleteSNP(Snp snp) {
		String hweId = String.valueOf(snp.getHwe().getId());

		String schemaname = InitServlet.dbSchemaname;

		String sql1 = "DELETE FROM "+schemaname+".DISCORDANCE WHERE snp_fk = " + snp.getId();
		String sql2 = "DELETE FROM "+schemaname+".HAPMAP WHERE snp_fk = " + snp.getId();
		String sql3 = "DELETE FROM "+schemaname+".GENOTYPE WHERE snp_fk = " + snp.getId();
		String sql4 = "DELETE FROM "+schemaname+".SNP WHERE id = " + snp.getId();
		String sql5 = "DELETE FROM "+schemaname+".HWE WHERE id = " + hweId;

		Session session = WorkflowDAO.beginTransaction();
		session.createSQLQuery(sql1).executeUpdate();
		session.createSQLQuery(sql2).executeUpdate();
		session.createSQLQuery(sql3).executeUpdate();
		session.createSQLQuery(sql4).executeUpdate();
		session.createSQLQuery(sql5).executeUpdate();

		WorkflowDAO.commitTransaction();
	}

	public static LinkedList<String[]> getDeleteOverview(LinkedList<ExampleHolder> dataList, String studyId){
		LinkedList<String[]> overview = new LinkedList<String[]>();

		Study study = DatabaseUtils.get(Study.class, Integer.parseInt(studyId));
		Set<Sample> samples = study.getSamples();
		HashMap<String, Integer> tmpHolder = new HashMap<String, Integer>();
		
		for(ExampleHolder eh : dataList){
		    String sampleName = eh.getSampleName();
			Sample exSample = new Sample();
			exSample.setSampleId(sampleName);

			if (!samples.contains(exSample)) {
				int tmpCounter = 1;
				if (tmpHolder.containsKey(sampleName)) {
					tmpCounter = tmpHolder.get(sampleName);
					++tmpCounter;
				}

				tmpHolder.put(sampleName, tmpCounter);
			}
		}

		for (String call : tmpHolder.keySet()) {
			String[] tmp = new String[2];
			tmp[0] = call;
			tmp[1] = String.valueOf(tmpHolder.get(call));

			overview.add(tmp);
		}

		return overview;
	}

	public static LinkedList<String[]> getDeleteOverview(LinkedList<ExampleHolder> dataList, Study study){
		LinkedList<String[]> overview = new LinkedList<String[]>();

		Set<Sample> samples = study.getSamples();
		HashMap<String, Integer> tmpHolder = new HashMap<String, Integer>();

		for(ExampleHolder eh : dataList){
			String sampleName = eh.getSampleName();
			Sample exSample = new Sample();
			exSample.setSampleId(sampleName);

			if (!samples.contains(exSample)) {
				int tmpCounter = 1;
				if (tmpHolder.containsKey(sampleName)) {
					tmpCounter = tmpHolder.get(sampleName);
					++tmpCounter;
				}

				tmpHolder.put(sampleName, tmpCounter);
			}
		}

		for (String call : tmpHolder.keySet()) {
			String[] tmp = new String[2];
			tmp[0] = call;
			tmp[1] = String.valueOf(tmpHolder.get(call));

			overview.add(tmp);
		}

		return overview;
	}

	public static void deleteSampleIds(LinkedList<ExampleHolder> sampleList, LinkedList<String> sampleIds) {
		LinkedList<ExampleHolder> deleteList = new LinkedList<ExampleHolder>();

		for (ExampleHolder exampleHolder : sampleList) {
			for (String sampleId : sampleIds) {
				if (sampleId.equalsIgnoreCase(exampleHolder.getSampleName()))
					deleteList.add(exampleHolder);
			}
		}

		sampleList.removeAll(deleteList);
	}

	public static void replaceGenoTypes(LinkedList<ExampleHolder> exampleList, HashMap<String, String> repl) {
		for (ExampleHolder ex : exampleList) {
			String call = ex.getCall();
			String str = repl.get(call);

			if (call.equalsIgnoreCase("Undetermined"))
				ex.setCall("");
			else if (str != null)
				ex.setCall(str);
		}
	}

	public static void mergeList(LinkedList<ExampleHolder> sampleList, String studyId) {
		Study study = DatabaseUtils.get(Study.class, Integer.parseInt(studyId));
		LinkedList<ExampleHolder> tmpList = new LinkedList<ExampleHolder>();

		for (Sample sample : study.getSamples()) {
			if(!sampleList.contains(sample)) {
				ExampleHolder tmp = new ExampleHolder();
				tmp.setSampleName(sample.getSampleId());
				tmp.setCall("");
				tmp.setFileName("");
				tmp.setWell("");
				tmpList.add(tmp);
			}
		}

		sampleList.addAll(tmpList);
	}

	public static void mergeList(LinkedList<ExampleHolder> sampleList, Study study) {
		LinkedList<ExampleHolder> tmpList = new LinkedList<ExampleHolder>();

		for (Sample sample : study.getSamples()) {
			if(!sampleList.contains(sample)) {
				ExampleHolder tmp = new ExampleHolder();
				tmp.setSampleName(sample.getSampleId());
				tmp.setCall("");
				tmp.setFileName("");
				tmp.setWell("");

				tmpList.add(tmp);
			}
		}

		sampleList.addAll(tmpList);
	}

	public static Snp getSnp(String snpId) {
		return DatabaseUtils.get(Snp.class, Integer.parseInt(getInteger(snpId)));
	}

	public static SnpContainer calculate(LinkedList<ExampleHolder> data, String studyId, String snpName, String snpComment, String snpInsertedBy, String snpDirectory, HashMap<String, String> repMap, String hapmapName) {
		Study study = DatabaseUtils.get(Study.class, Integer.parseInt(studyId));
		Snp snp = new Snp();
		snp.setName(snpName);
		snp.setHapmapLookup(hapmapName);
		snp.setComment(snpComment);
		snp.setInsertedBy(snpInsertedBy);
		snp.setCreatedWhen(new Date());
		snp.setStudy(study);
		snp.setNumMultiple(study.getNumMultiple());
		snp.setSex(0); //default value

		Collections.sort(data);
		int equal = getEqualExamples(data);

		sortOutEqualExamples(data);


		
		int diff = getDiffExamples(data);
		LinkedList<ExampleHolder> listDiscordant = sortOutDiffExamples(data);

		LinkedList<ExampleHolder> listReplicates = sortOutExamples(data);
		
		logger.debug("sizes listNonDisc " + listDiscordant.size() +  " listDisc " + listReplicates.size() + " data " + data.size() );
		logger.debug("Diff + "+ diff + " equals " +equal);
	
		HashSet<Discordance> discs = getDiscordance(listReplicates, true);
		
		discs.addAll(getDiscordance(listDiscordant, false));
		
	resetCalls(data, listDiscordant);

	
	LinkedList<String> replacements = new LinkedList<String>(repMap.values());
	while (replacements.remove(""));
	Collections.sort(replacements);

		String nameHomo1 = extractHomo(replacements);
		replacements.remove(nameHomo1);
		String nameHomo2 = extractHomo(replacements);
		replacements.remove(nameHomo2);
		String nameHetero = getHetero(replacements);
	
		int numHomo1 = getNum(nameHomo1, data);
		int numHomo2 = getNum(nameHomo2, data);
		int numHetero = getNum(nameHetero, data);

		
		snp.setNumHetero(numHetero);
		snp.setNameHetero(nameHetero);

		double sumAll = data.size();
		snp.setNumSamples(data.size());
		double numMulti = study.getNumMultiple();

		snp.setProcQcSamples((numMulti/sumAll)*100);

		double numCalled = equal + diff;

		snp.setProcQc((numCalled*100d/sumAll));

		snp.setNumDisc(diff);
		
		if (numCalled!=0)
			snp.setProcDiscRate((diff*100d)/numCalled);
		else
			snp.setProcDiscRate(0);
		
		int aa = 0;
		if(numHomo1 > numHomo2) {
			aa = numHomo1;
			snp.setNameHomo1(nameHomo1);
			snp.setNumHomo1(numHomo1);
		} else {
			aa = numHomo2;
			snp.setNameHomo1(nameHomo2);
			snp.setNumHomo1(numHomo2);
		}
		int ab = numHetero;
		int bb = 0;
		if (numHomo1 > numHomo2) {       //was > error 88 vs. 88
			bb = numHomo2;
			snp.setNameHomo2(nameHomo2);
			snp.setNumHomo2(numHomo2);
		} else {
			bb = numHomo1;
			snp.setNameHomo2(nameHomo1);
			snp.setNumHomo2(numHomo1);
		}

		snp.setProcCallRate(((aa+ab+bb)*100d)/sumAll);
	
		Double procMaf = calcMaf(aa, ab, bb);
	
		if (!procMaf.isNaN()){
			snp.setProcMaf(procMaf);
		}
		else
			snp.setProcMaf(0.0);
		
	
		Hwe hwe = calcHwe(aa, bb, ab);

		
	/*	System.out.println("HWE: " + hwe.getChiSquare() + " " +hwe.getExact());
		System.out.println("procMaf: " + procMaf);
		System.out.println("setProcCallRate: " + (((aa+ab+bb)*100d)/sumAll));
		System.out.println("diff: " + diff);
		System.out.println("procDiscRate: " + (diff*100d)/numCalled +  " numcalled " + numCalled + " eq " + equal + " diff " +diff);
	*/	
		snp.setHwe(hwe);

snp.setDirectory(snpDirectory);


SnpContainer snpc = new SnpContainer();
snpc.setSnp(snp);
snpc.setData(data);
snpc.setDisc(discs);

return snpc;

}

public static int writeSnpToDB(SnpContainer snpc) {
		int snpId = DatabaseUtils.add(snpc.getSnp());
		setSnpForDiscList(snpId, snpc.getDisc());
		DatabaseUtils.add(snpc.getDisc(), true);
		LinkedList<Genotype> genotypes = createGenotypes(snpId, snpc.getData());
		System.out.println("Genotypes " +genotypes.size() + " " + snpc.getSnp().getName());
		
		DatabaseUtils.add(genotypes, true);
		
		return snpId;
}

	public static Snp getSnpById( String snpId) {
		Snp snp = null;
		if (snpId.contains("_")){
			snpId=snpId.split("_")[1];
		}
		try{
		snp=	DatabaseUtils.get(Snp.class, Integer.parseInt(snpId.trim()));
		}catch (Exception e) {
			logger.error("SNP id wrong " + snpId);
		}
		return snp;
	}

	public static byte[] getGenotypeDownload(final Snp snp, final Locale locale){
		ResourceBundle labels = ResourceBundle.getBundle("workflow", locale);
		LinkedList<Genotype> genotypes = new LinkedList<Genotype>(snp.getGenotypes());
		Collections.sort(genotypes);

		StringBuffer line = new StringBuffer();
		line.append("\"ID\"");line.append(TAB);
		line.append("\""+labels.getString("lbGenotype")+"\"");line.append(CARRIAGE);

		for(Genotype gen : genotypes){
			line.append("\""+gen.getSampleId()+"\"");line.append(TAB);

			line.append("\""+gen.getGenotype()+"\"");line.append(CARRIAGE);
		}

		return line.toString().getBytes();
	}

	public static byte[] getHapmapDownload(final Snp snp, final Locale locale){
		ResourceBundle labels = ResourceBundle.getBundle("workflow", locale);
		LinkedList<Genotype> genotypes = new LinkedList<Genotype>(snp.getGenotypes());
		Collections.sort(genotypes);

		StringBuffer line = new StringBuffer();
		line.append("\""+labels.getString("lbPopulation")+"\"");line.append(TAB);
		line.append("\""+snp.getNameHomo1()+"\"");line.append(TAB);
		line.append("\""+snp.getNameHetero()+"\"");line.append(TAB);
		line.append("\""+snp.getNameHomo2()+"\"");line.append(TAB);
		line.append("\""+labels.getString("lbPValue")+"\"");line.append(CARRIAGE);

		List<Hapmap> sorted = new LinkedList<Hapmap>(snp.getHapmaps());
		Collections.sort(sorted,  new Comparator<Hapmap>() {
	        public int compare(Hapmap h1, Hapmap h2) {
	            return h1.getPopulation().compareToIgnoreCase(h2.getPopulation());
	        }
	    });

		DecimalFormat f3 = new DecimalFormat("#0.000");

		line.append("\"OWN DATA\"");line.append(TAB);
		line.append("\""+f3.format((double)snp.getNumHomo1()/(double)snp.getNumSamples()) + " | " + snp.getNumHomo1() +"\"");line.append(TAB);
		line.append("\""+f3.format((double)snp.getNumHetero()/(double)snp.getNumSamples()) + " | " + snp.getNumHetero() +"\"");line.append(TAB);
		line.append("\""+f3.format((double)snp.getNumHomo2()/(double)snp.getNumSamples()) + " | " + snp.getNumHomo2() +"\"");line.append(TAB);
		line.append("\"\"");line.append(CARRIAGE);

		DecimalFormat f = new DecimalFormat("0.##E0");
		for(Hapmap hm : sorted){
			line.append("\""+hm.getPopulation()+"\"");line.append(TAB);
			line.append("\""+hm.getHomoFreq1() + " | " + hm.getHomo1() +"\"");line.append(TAB);
			line.append("\""+hm.getHeteroFreq() + " | " + hm.getHetero() +"\"");line.append(TAB);
			line.append("\""+hm.getHomoFreq2() + " | " + hm.getHomo2() +"\"");line.append(TAB);
			if (hm.getChi() == 0.0){
				line.append("\"<2.06E-9\"");line.append(CARRIAGE);
			} else {
				line.append("\""+f.format(hm.getChi())+"\"");line.append(CARRIAGE);
			}
		}

		return line.toString().getBytes();
	}

	public static byte[] getSummaryDownload(final Snp snp, final Locale locale){
		ResourceBundle labels = ResourceBundle.getBundle("workflow", locale);
		LinkedList<Genotype> genotypes = new LinkedList<Genotype>(snp.getGenotypes());
		Collections.sort(genotypes);
		Hwe hwe = snp.getHwe();

		StringBuffer line = new StringBuffer();
		line.append("\""+labels.getString("lbGeneral")+"\"");line.append(TAB);line.append("");line.append(CARRIAGE);
		line.append("\""+labels.getString("lbNameSnp")+"\"");line.append(TAB);line.append("\""+snp.getName()+"\"");line.append(CARRIAGE);
		line.append("\""+labels.getString("lbDateSnp")+"\"");line.append(TAB);line.append("\""+snp.getCreatedWhen()+"\"");line.append(CARRIAGE);
		line.append("\""+labels.getString("lbCreator")+"\"");line.append(TAB);line.append("\""+snp.getInsertedBy()+"\"");line.append(CARRIAGE);
		line.append("\""+labels.getString("lbComment")+"\"");line.append(TAB);line.append("\""+snp.getComment()+"\"");line.append(CARRIAGE);
		line.append(CARRIAGE);
		line.append("\""+labels.getString("lbSampleCount")+"\"");line.append(TAB);line.append("\""+snp.getNumSamples()+"\"");line.append(CARRIAGE);
		line.append("\""+labels.getString("lbQcSampleTotal")+"\"");line.append(TAB);line.append("\""+snp.getProcDiscRate()+"\"");line.append(CARRIAGE);
		line.append("\""+labels.getString("lbCallrate")+"\"");line.append(TAB);line.append("\""+snp.getProcCallRate()+"\"");line.append(CARRIAGE);
		line.append("\""+labels.getString("lbMaf")+"\"");line.append(TAB);line.append("\""+snp.getProcMaf()+"\"");line.append(CARRIAGE);
		line.append(CARRIAGE);
		line.append("\""+labels.getString("lbQualityControl")+"\"");line.append(TAB);line.append("");line.append(CARRIAGE);
		line.append("\""+labels.getString("lbDiscordanceRate")+"\"");line.append(TAB);line.append("\""+snp.getProcQcSamples()+"\"");line.append(CARRIAGE);
		line.append("\""+labels.getString("lbDiscordance")+"\"");line.append(TAB);line.append("\""+snp.getNumDisc()+"\"");line.append(CARRIAGE);
		line.append("\""+labels.getString("lbQcSamplesCalled")+"\"");line.append(TAB);line.append("\""+snp.getProcQc()+"\"");line.append(CARRIAGE);
		line.append(CARRIAGE);
		line.append("\""+labels.getString("lbHwe")+"\"");line.append(TAB);line.append("\""+labels.getString("lbObserved")+"\"");line.append(TAB);line.append("\""+labels.getString("lbExpected")+"\"");line.append(CARRIAGE);
		line.append("\""+snp.getNameHomo1()+"\"");line.append(TAB);line.append("\""+hwe.getHomoObserved1()+"\"");line.append(TAB);line.append("\""+hwe.getHomoExpected1()+"\"");line.append(CARRIAGE);
		line.append("\""+snp.getNameHetero()+"\"");line.append(TAB);line.append("\""+hwe.getHeteroObserved()+"\"");line.append(TAB);line.append("\""+hwe.getHeteroExpected()+"\"");line.append(CARRIAGE);
		line.append("\""+snp.getNameHomo2()+"\"");line.append(TAB);line.append("\""+hwe.getHomoObserved2()+"\"");line.append(TAB);line.append("\""+hwe.getHomoExpected2()+"\"");line.append(CARRIAGE);
		line.append(CARRIAGE);
		DecimalFormat f = new DecimalFormat("0.##E0");
		line.append("\""+labels.getString("lbHweTest")+"\"");line.append(TAB);line.append("\""+f.format(hwe.getChiSquare())+"\"");line.append(CARRIAGE);
//		line.append("\"Exact\"");line.append(TAB);line.append("\""+hwe.getExact()+"\"");line.append(CARRIAGE);
		line.append(CARRIAGE);
		line.append("\""+labels.getString("lbDiscordance")+"\"");line.append(TAB);line.append("");line.append(CARRIAGE);
		for (Discordance disc : snp.getDiscordances()) {
			if(!disc.isDiscordant() && disc.getSampleId() != null && !disc.getSampleId().isEmpty())
				line.append("\""+disc.getSampleId()+"\"");line.append(TAB);line.append("");line.append(CARRIAGE);
		}
		line.append(CARRIAGE);
		line.append("\""+labels.getString("lbMissing")+"\"");line.append(TAB);line.append("");line.append(CARRIAGE);
		for (Discordance disc : snp.getDiscordances()) {
			if(disc.isDiscordant())
				line.append("\""+disc.getSampleId()+"\"");line.append(TAB);line.append("");line.append(CARRIAGE);
		}

		line.append(CARRIAGE);

		return line.toString().getBytes();
	}

	public static void writeExcelSummary(final Snp snp, OutputStream out, final Locale locale) throws IOException {
		ResourceBundle labels = ResourceBundle.getBundle("workflow", locale);
		WritableCellFormat integerFormat = new WritableCellFormat(NumberFormats.INTEGER);
		WritableCellFormat floatFormat = new WritableCellFormat(NumberFormats.FLOAT);
		WritableCellFormat floatFormat2 = new WritableCellFormat(NumberFormats.EXPONENTIAL);
		try {
			WritableWorkbook wb = Workbook.createWorkbook(out);
			WritableSheet sheet = wb.createSheet(labels.getString("lbSummary"), 0);
			
			sheet.setColumnView(0, 30);
			sheet.setColumnView(1, 15);
			sheet.setColumnView(2, 15);
			Label l1 = new Label(0, 0, labels.getString("lbGeneral"));
			sheet.addCell(l1);

			Label l2 = new Label(0, 1, labels.getString("lbNameSnp"));
			sheet.addCell(l2);
			Label l3 = new Label(1, 1, snp.getName());
			sheet.addCell(l3);

			Label l4 = new Label(0, 2, labels.getString("lbDateSnp"));
			sheet.addCell(l4);
			DateTime d1 = new DateTime(1, 2, snp.getCreatedWhen());
			sheet.addCell(d1);

			Label l5 = new Label(0, 3, labels.getString("lbCreator"));
			sheet.addCell(l5);
			Label l6 = new Label(1, 3, snp.getInsertedBy());
			sheet.addCell(l6);

			Label l7 = new Label(0, 4, labels.getString("lbComment"));
			sheet.addCell(l7);
			Label l8 = new Label(1, 4, snp.getComment());
			sheet.addCell(l8);

			Label l9 = new Label(0, 6, labels.getString("lbSampleCount") + " [n]");
			sheet.addCell(l9);
			jxl.write.Number number = new jxl.write.Number(1, 6, snp.getNumSamples(), integerFormat);
			sheet.addCell(number);

			Label l10 = new Label(0, 7, labels.getString("lbQcSampleTotal2"));
			sheet.addCell(l10);
			jxl.write.Number l11 = new jxl.write.Number(1, 7, snp.getProcQcSamples(), floatFormat);
			sheet.addCell(l11);

			Label l12 = new Label(0, 8, labels.getString("lbCallrate") + " [%]");
			sheet.addCell(l12);
			jxl.write.Number l13 = new jxl.write.Number(1, 8, snp.getProcCallRate(), floatFormat);
			sheet.addCell(l13);

			Label l14 = new Label(0, 9, labels.getString("lbMaf") + " [%]");
			sheet.addCell(l14);
			jxl.write.Number l15 = new jxl.write.Number(1, 9, snp.getProcMaf(), floatFormat);
			sheet.addCell(l15);
		if (snp.getSex()>0){
			Label l15s = new Label(0, 10, labels.getString("lbSexSnpDiscordant") + " [n]");
			sheet.addCell(l15s);
			jxl.write.Number l15n = new jxl.write.Number(1, 10, snp.getSexsnp().size(), integerFormat);
			sheet.addCell(l15n);
			}

			Label l16 = new Label(0, 11, labels.getString("lbQualityControl"));
			sheet.addCell(l16);

			Label l17 = new Label(0, 12, labels.getString("lbDiscordanceRate") + " [%]");
			sheet.addCell(l17);
			jxl.write.Number l18 = new jxl.write.Number(1, 12, snp.getProcDiscRate(), floatFormat);
			sheet.addCell(l18);

			Label l19 = new Label(0, 13, labels.getString("lbDiscordance") + " [n]");
			sheet.addCell(l19);
			jxl.write.Number l20 = new jxl.write.Number(1, 13, snp.getNumDisc(), floatFormat);
			sheet.addCell(l20);

			Label l21 = new Label(0, 14, labels.getString("lbQcSamplesCalled2") + " [%]");
			sheet.addCell(l21);
			jxl.write.Number l22 = new jxl.write.Number(1, 14, snp.getProcQc(), floatFormat);
			sheet.addCell(l22);

			Label l23 = new Label(0, 16, labels.getString("lbHwe"));
			sheet.addCell(l23);
			Label l24 = new Label(1, 16, labels.getString("lbObserved2") + " [n]");
			sheet.addCell(l24);
			Label l25 = new Label(2, 16, labels.getString("lbExpected2") + " [n]");
			sheet.addCell(l25);

			Hwe hwe = snp.getHwe();
			Label l26 = new Label(0, 17, snp.getNameHomo1());
			sheet.addCell(l26);
			jxl.write.Number l29 = new jxl.write.Number(1, 17, hwe.getHomoObserved1(), integerFormat);
			sheet.addCell(l29);
			jxl.write.Number l32 = new jxl.write.Number(2, 17, hwe.getHomoExpected1(), floatFormat);
			sheet.addCell(l32);

			Label l27 = new Label(0, 18, snp.getNameHetero());
			sheet.addCell(l27);
			jxl.write.Number l30 = new jxl.write.Number(1, 18, hwe.getHeteroObserved(), integerFormat);
			sheet.addCell(l30);
			jxl.write.Number l33 = new jxl.write.Number(2, 18, hwe.getHeteroExpected(), floatFormat);
			sheet.addCell(l33);

			Label l28 = new Label(0, 19, snp.getNameHomo2());
			sheet.addCell(l28);
			jxl.write.Number l31 = new jxl.write.Number(1, 19, hwe.getHomoObserved2(), integerFormat);
			sheet.addCell(l31);
			jxl.write.Number l34 = new jxl.write.Number(2, 19, hwe.getHomoExpected2(), floatFormat);
			sheet.addCell(l34);

			Label l35 = new Label(0, 21, labels.getString("lbHweTest"));
			sheet.addCell(l35);
			DecimalFormat f = new DecimalFormat("0.##E0");
			jxl.write.Number l36 = new jxl.write.Number(2, 21, hwe.getChiSquare(),floatFormat2);
			sheet.addCell(l36);

//			Label l37 = new Label(0, 22, "Fisher Exact");
//			sheet.addCell(l37);
//			jxl.write.Number l38 = new jxl.write.Number(1, 22, hwe.getExact(), fourFormat);
//			sheet.addCell(l38);

			WritableSheet sheet1 = wb.createSheet(labels.getString("lbGenotypes"), 1);

			LinkedList<Genotype> genotypes = new LinkedList<Genotype>(snp.getGenotypes());
			Collections.sort(genotypes);

			Label h1 = new Label(0, 0, "ID");
			sheet1.addCell(h1);
			Label h2 = new Label(1, 0, snp.getName());
			sheet1.addCell(h2);

			int i = 1;
			for(Genotype gen : genotypes){
				Label tmp1 = new Label(0, i, gen.getSampleId());
				Label tmp2 = new Label(1, i, gen.getGenotype());
				sheet1.addCell(tmp1);
				sheet1.addCell(tmp2);
				++i;
			}

			WritableSheet sheet2 = wb.createSheet("HapMap comparison (R27)", 2);

			Label h3 = new Label(0, 0, labels.getString("lbPopulation"));
			sheet2.addCell(h3);
			Label h4 = new Label(1, 0, snp.getNameHomo1());
			sheet2.addCell(h4);
			Label h5 = new Label(2, 0, snp.getNameHetero());
			sheet2.addCell(h5);
			Label h6 = new Label(3, 0, snp.getNameHomo2());
			sheet2.addCell(h6);
			Label h7 = new Label(4, 0, labels.getString("lbPValue"));
			sheet2.addCell(h7);

			List<Hapmap> sorted = new LinkedList<Hapmap>(snp.getHapmaps());
			Collections.sort(sorted,  new Comparator<Hapmap>() {
		        public int compare(Hapmap h1, Hapmap h2) {
		            return h1.getPopulation().compareToIgnoreCase(h2.getPopulation());
		        }
		    });

			DecimalFormat f3 = new DecimalFormat("#0.000");

			Label l38 = new Label(0, 1, "OWN DATA");
			sheet2.addCell(l38);
			Label l39 = new Label(1, 1, f3.format((double)snp.getNumHomo1()/(double)snp.getNumSamples()) + " | " + String.valueOf(snp.getNumHomo1()));
			sheet2.addCell(l39);
			Label l40 = new Label(2, 1, f3.format((double)snp.getNumHetero()/(double)snp.getNumSamples()) + " | " + String.valueOf(snp.getNumHetero()));
			sheet2.addCell(l40);
			Label l41 = new Label(3, 1, f3.format((double)snp.getNumHomo2()/(double)snp.getNumSamples()) + " | " + String.valueOf(snp.getNumHomo2()));
			sheet2.addCell(l41);
			Label l42 = new Label(4, 1, "");
			sheet2.addCell(l42);

			int j = 2;
			for(Hapmap hm : sorted){
				Label tmp1 = new Label(0, j, hm.getPopulation());
				sheet2.addCell(tmp1);
				Label tmp2 = new Label(1, j, String.valueOf(hm.getHomoFreq1()) + " | " + String.valueOf(hm.getHomo1()));
				sheet2.addCell(tmp2);
				Label tmp3 = new Label(2, j, String.valueOf(hm.getHeteroFreq()) + " | " + String.valueOf(hm.getHetero()));
				sheet2.addCell(tmp3);
				Label tmp4 = new Label(3, j, String.valueOf(hm.getHomoFreq2()) + " | " + String.valueOf(hm.getHomo2()));
				sheet2.addCell(tmp4);
				Label tmp5 = new Label(4, j, hm.getChi() == 0 ? "<2.06E-9" : f.format(hm.getChi()));
				sheet2.addCell(tmp5);

				++j;
			}

			wb.write();
			wb.close();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (WriteException e) {
			e.printStackTrace();
		} finally {
			if(out != null)
				out.close();
		}
	}
	
	
	public static void writeExcelGenotypesOverview(final Vector<String> snpvector, OutputStream out, final Locale locale) throws IOException {
		ResourceBundle labels = ResourceBundle.getBundle("workflow", locale);
		
		try {
			WritableWorkbook wb = Workbook.createWorkbook(out);
			WritableSheet sheet1 = wb.createSheet(labels.getString("lbGenotypes"), 1);
		
			Label h1 = new Label(0, 0, "ID");
			sheet1.addCell(h1);

			for ( int i =0; i <snpvector.size(); i++){
				Snp 	snp = SNPManagement.getSnpById(snpvector.get(i));
				
				LinkedList<Genotype> genotypes = new LinkedList<Genotype>(snp.getGenotypes());
				Collections.sort(genotypes);
			Label h2 = new Label(1+i, 0, snp.getName());
			sheet1.addCell(h2);

			int j = 1;
			for(Genotype gen : genotypes){
				if (i==0)
					{
					Label tmp1 = new Label(0, j, gen.getSampleId());
					sheet1.addCell(tmp1);
					}
				Label tmp2 = new Label(1+i, j, gen.getGenotype());
				sheet1.addCell(tmp2);
				++j;
			}
			}

			wb.write();
			wb.close();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (WriteException e) {
			e.printStackTrace();
		} finally {
			if(out != null)
				out.close();
		}
	}
	
	
	public static void writeExcelSummaryTotal(final Vector<String> snpvector, OutputStream out, final Locale locale, boolean [] checkHapMap) throws IOException {
		ResourceBundle labels = ResourceBundle.getBundle("workflow", locale);
		WritableCellFormat floatFormat2 = new WritableCellFormat(NumberFormats.EXPONENTIAL);
		WritableCellFormat integerFormat = new WritableCellFormat(NumberFormats.INTEGER);
		WritableCellFormat floatFormat = new WritableCellFormat(NumberFormats.FLOAT);
		WritableCellFormat dateFormat = new WritableCellFormat ( new DateFormat ("dd.MM.yyyy"));
		
		try {
			WritableWorkbook wb = Workbook.createWorkbook(out);
			WritableSheet sheet = wb.createSheet(labels.getString("lbSummary"), 0);
			sheet.setColumnView(0, 30);
			
			 WritableCellFormat cellFormatYellow = new WritableCellFormat(NumberFormats.FLOAT);
	         cellFormatYellow.setBackground(Colour.YELLOW);
	         WritableCellFormat cellFormatYellowInt = new WritableCellFormat(NumberFormats.INTEGER);
	         cellFormatYellowInt.setBackground(Colour.YELLOW);
	         WritableCellFormat normalFormat = new WritableCellFormat();
	         normalFormat.setAlignment(jxl.format.Alignment.CENTRE);
	         
	          
	         WritableCellFormat cellFormatOrange = new WritableCellFormat(NumberFormats.EXPONENTIAL);
	         cellFormatOrange.setBackground(Colour.GOLD);
	          
	         WritableCellFormat cellFormatRed = new WritableCellFormat(NumberFormats.EXPONENTIAL);
	         cellFormatRed.setBackground(Colour.ORANGE);
			
			
			Label l1 = new Label(0, 0, labels.getString("lbNameSnp"));
			sheet.addCell(l1);

			Label l2 = new Label(0, 1, labels.getString("lbStudy"));
			sheet.addCell(l2);
		

			Label l4 = new Label(0, 2, labels.getString("lbDateSnp"));
			sheet.addCell(l4);
			
			Label l5 = new Label(0, 3, labels.getString("lbCreator"));
			sheet.addCell(l5);
			
			Label l7 = new Label(0, 4, labels.getString("lbComment"));
			sheet.addCell(l7);



			Label l9 = new Label(0, 5, labels.getString("lbSampleCount") + " [n]");
			sheet.addCell(l9);
		
			Label l10 = new Label(0, 6, labels.getString("lbQcSampleTotal2"));
			sheet.addCell(l10);
	
			Label l12 = new Label(0, 7, labels.getString("lbCallrate") + " [%]");
			sheet.addCell(l12);
	
			Label l14 = new Label(0, 8, labels.getString("lbMaf") + " [%]");
			sheet.addCell(l14);
			
			Label l15s = new Label(0, 9, labels.getString("lbSexSnpDiscordant") + " [n]");
			sheet.addCell(l15s);
	
			Label l17 = new Label(0, 10, labels.getString("lbDiscordanceRate") + " [%]");
			sheet.addCell(l17);
		
			Label l19 = new Label(0, 11, labels.getString("lbDiscordance") + " [n]");
			sheet.addCell(l19);
	
			Label l21 = new Label(0, 12, labels.getString("lbQcSamplesCalled2") + " [%]");
			sheet.addCell(l21);
			
			Label l35 = new Label(0, 13, labels.getString("lbHweTest"));
			sheet.addCell(l35);
			
			Label l23 = new Label(0, 14, "Genotype_AA");
			sheet.addCell(l23);
			Label l24 = new Label(0, 15, "Genotype_Aa");
			sheet.addCell(l24);
			Label l25 = new Label(0, 16, "Genotype_aa");
			sheet.addCell(l25);
			Label l26 = new Label(0, 17, "Genotype_AA (observed)");
			sheet.addCell(l26);
			Label l27 = new Label(0, 18, "Genotype_Aa (observed)");
			sheet.addCell(l27);
			Label l28 = new Label(0, 19, "Genotype_aa (observed)");
			sheet.addCell(l28);
			
			Label l29 = new Label(0, 20, "Genotype_AA (expected)");
			sheet.addCell(l29);
			Label l30 = new Label(0, 21, "Genotype_Aa (expected)");
			sheet.addCell(l30);
			Label l31 = new Label(0, 22, "Genotype_aa (expected)");
			sheet.addCell(l31);
			Label l31hap = new Label(0, 23, labels.getString("lbHapMap"));
			sheet.addCell(l31hap);
			
			 WritableCellFormat wcf = new WritableCellFormat();
			 wcf.setAlignment(Alignment.CENTRE);
			 cellFormatOrange.setAlignment(Alignment.CENTRE);
			 cellFormatYellow.setAlignment(Alignment.CENTRE);
			 cellFormatYellowInt.setAlignment(Alignment.CENTRE);
			 cellFormatRed.setAlignment(Alignment.CENTRE);
			 dateFormat.setAlignment(Alignment.CENTRE);
			 integerFormat.setAlignment(Alignment.CENTRE);
			 floatFormat.setAlignment(Alignment.CENTRE);
			 floatFormat2.setAlignment(Alignment.CENTRE);
		
			 for ( int i =0; i <snpvector.size(); i++){

				Snp snp = SNPManagement.getSnpById(snpvector.get(i));
					Label h1 = new Label(1+i, 0, snp.getName() + "_"+snp.getDirectory(), wcf);   //SNP_id
					sheet.addCell(h1);
					Label l8 = new Label(1+i, 1, snp.getStudy().getName(), wcf);
					sheet.addCell(l8);
					DateTime d1 = new DateTime(1+i, 2, snp.getCreatedWhen(), dateFormat);
					sheet.addCell(d1);
					Label l6 = new Label(1+i, 3, snp.getInsertedBy(), wcf);
					sheet.addCell(l6);
					Label l8b = new Label(1+i, 4, snp.getComment());
					sheet.addCell(l8b);
					jxl.write.Number number = new jxl.write.Number(1+i, 5, snp.getNumSamples(), integerFormat);
					sheet.addCell(number);
					jxl.write.Number l11 = new jxl.write.Number(1+i, 6, snp.getProcQcSamples(), floatFormat);
					sheet.addCell(l11);
					jxl.write.Number l13 = new jxl.write.Number(1+i, 7, snp.getProcCallRate(), floatFormat);
					if (snp.getProcCallRate()<95)
						l13.setCellFormat(cellFormatYellow);
					sheet.addCell(l13);
					jxl.write.Number l15 = new jxl.write.Number(1+i, 8, snp.getProcMaf(), floatFormat);
					sheet.addCell(l15);
					if (snp.getSex()>0){
						jxl.write.Number l15n = new jxl.write.Number(1+i, 9, snp.getSexsnp().size(), integerFormat);
						if (snp.getSexsnp().size()>0)
							l15n.setCellFormat(cellFormatYellowInt);
						sheet.addCell(l15n);
					}
					else{
						Label l15n = new Label(1+i, 9, "", wcf);
						sheet.addCell(l15n);
					}
					
					jxl.write.Number l18 = new jxl.write.Number(1+i, 10, snp.getProcDiscRate(), floatFormat);
					sheet.addCell(l18);
					jxl.write.Number l20 = new jxl.write.Number(1+i, 11, snp.getNumDisc(), floatFormat);
					if (snp.getNumDisc()>0)
						l20.setCellFormat(cellFormatYellowInt);
					sheet.addCell(l20);
					jxl.write.Number l22 = new jxl.write.Number(1+i, 12, snp.getProcQc(), floatFormat);
					sheet.addCell(l22);
					Hwe hwe = snp.getHwe();
					jxl.write.Number l36 = new jxl.write.Number(1+i, 13, hwe.getChiSquare(),floatFormat2);
					if (hwe.getChiSquare()<0.05)
					{
						if (hwe.getChiSquare()<0.000001)
						{
							l36.setCellFormat(cellFormatRed);
						}
						else{
							l36.setCellFormat(cellFormatOrange);
						}
					}
					
					
					sheet.addCell(l36);
				
					Label l37 = new Label(1+i, 14, snp.getNameHomo1(), wcf);
					sheet.addCell(l37);
					Label l39 = new Label(1+i, 15, snp.getNameHetero(), wcf);
					sheet.addCell(l39);
					Label l42 = new Label(1+i, 16, snp.getNameHomo2(), wcf);
					sheet.addCell(l42);
					
					jxl.write.Number l38 = new jxl.write.Number(1+i, 17, hwe.getHomoObserved1(), integerFormat);
					sheet.addCell(l38);
					jxl.write.Number l40 = new jxl.write.Number(1+i, 18, hwe.getHeteroObserved(), integerFormat);
					sheet.addCell(l40);
					jxl.write.Number l43 = new jxl.write.Number(1+i, 19, hwe.getHomoObserved2(), integerFormat);
					sheet.addCell(l43);
					
					jxl.write.Number l32 = new jxl.write.Number(1+i, 20, hwe.getHomoExpected1(), floatFormat);
					sheet.addCell(l32);
					jxl.write.Number l41 = new jxl.write.Number(1+i, 21, hwe.getHeteroExpected(), floatFormat);
					sheet.addCell(l41);
					jxl.write.Number l44 = new jxl.write.Number(1+i, 22, hwe.getHomoExpected2(), floatFormat);
					sheet.addCell(l44);
					 
					  CellView c = new CellView();
					  c.setAutosize(true);
					  sheet.setColumnView(i+1, c);
					  
						List<Hapmap> hapmap = new LinkedList<Hapmap>(snp.getHapmaps());
						Collections.sort(hapmap,  new Comparator<Hapmap>() {
					        public int compare(Hapmap h1, Hapmap h2) {
					            return h1.getPopulation().compareToIgnoreCase(h2.getPopulation());
					        }
					    });
						
						int j=0;
						

						for (int k =0; k < arrayHapmap.length; k++ ){
							if (checkHapMap[k]){
							
							for(Hapmap hm : hapmap){
							
								if (hm.getPopulation().contains(arrayHapmap[k])){
									if (i==0){
										
										Label h7 = new Label(0, 24+j*4, hm.getPopulation() + " ("+labels.getString("lbPValue1")+")");
										Label h4 = new Label(0, 24+j*4+1, "AA");
										sheet.addCell(h4);
										Label h5 = new Label(0, 24+j*4+2, "Aa");
										sheet.addCell(h5);
										Label h6 = new Label(0, 24+j*4+3, "aa");
										sheet.addCell(h6);
										sheet.addCell(h7);
									}
									
									jxl.write.Number tmp1= new jxl.write.Number(1+i, 24+j*4, hm.getChi(), floatFormat); //p-value
									Label tmp2 = new Label(1+i, 24+j*4+1, String.valueOf(hm.getHomoFreq1()) + " | " + String.valueOf(hm.getHomo1()), normalFormat);
									sheet.addCell(tmp2);
									Label tmp3 = new Label(1+i, 24+j*4+2, String.valueOf(hm.getHeteroFreq()) + " | " + String.valueOf(hm.getHetero()), normalFormat);
									sheet.addCell(tmp3);
									Label tmp4 = new Label(1+i, 24+j*4+3, String.valueOf(hm.getHomoFreq2()) + " | " + String.valueOf(hm.getHomo2()), normalFormat);
									sheet.addCell(tmp4);
									if (hm.getChi()<0.05)
									{
										if (hm.getChi()<0.000001)
										{
											tmp1.setCellFormat(cellFormatRed);
										}
										else{
											tmp1.setCellFormat(cellFormatOrange);
										}
									}
									 sheet.addCell(tmp1);
									 break;
								}
							}
							
							++j;
						}
				
						}
			 }

			wb.write();
			wb.close();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (WriteException e) {
			e.printStackTrace();
		} finally {
			if(out != null)
				out.close();
		}
	}
	

	public static void getAllZipDownload(OutputStream out, Snp snp, boolean excel, boolean genotypes, boolean summary, boolean hapmap, final Locale locale) {
		ZipOutputStream zipStream = new ZipOutputStream(out);

		try {
			if(summary){
				byte sumData[] = getSummaryDownload(snp, locale);

				zipStream.putNextEntry(new ZipEntry(snp.getName() + "_QC.tsv"));
				zipStream.write(sumData);
				zipStream.closeEntry();
			}
			if(excel){
				ByteArrayOutputStream baos = new ByteArrayOutputStream();
				writeExcelSummary(snp, baos, locale);

				zipStream.putNextEntry(new ZipEntry(snp.getName() + ".xls"));
				zipStream.write(baos.toByteArray());
				zipStream.closeEntry();
			}
			if(genotypes){
				byte genoData[] = getGenotypeDownload(snp, locale);

				zipStream.putNextEntry(new ZipEntry(snp.getName() + ".tsv"));
				zipStream.write(genoData);
				zipStream.closeEntry();
			}
			if(hapmap){
				byte genoData[] = getHapmapDownload(snp, locale);

				zipStream.putNextEntry(new ZipEntry(snp.getName() + "_HapMap.tsv"));
				zipStream.write(genoData);
				zipStream.closeEntry();
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				zipStream.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	private static LinkedList<Genotype> createGenotypes(int snpId, LinkedList<ExampleHolder> data) {
		LinkedList<Genotype> genotypes = new LinkedList<Genotype>();
		Snp snp = new Snp();
		snp.setId(snpId);

		for (ExampleHolder ex : data) {
			Genotype gen = new Genotype();
			gen.setSampleId(ex.getSampleName());
			gen.setGenotype(cleanCallName(ex.getCall()));
			gen.setSnp(snp);
			genotypes.add(gen);
		}
System.out.println("genotypes size" + genotypes.size());
		return genotypes;
	}


	private static String extractHomo(LinkedList<String> reps){
		for (String rep : reps) {
			String a = rep.substring(0, 1);
			String b = rep.substring(1, 2);
			if (a.equals(b)) {
				return rep;
			}
		}
		return "";
	}

	private static String getHetero(LinkedList<String> reps){
		for (String rep : reps) {
			String a = rep.substring(0, 1);
			String b = rep.substring(1, 2);
			if (!a.equals(b)) {
				return rep;
			}
		}

		return "";
	}

	private static void setSnpForDiscList(int snpId, HashSet<Discordance> discs) {
		Snp snp = new Snp();
		snp.setId(snpId);

		for (Discordance disc : discs) {
			disc.setSnp(snp);
		}
	}

	private static HashSet<Discordance> getDiscordance(LinkedList<ExampleHolder> list, boolean discordance) {
		HashSet<Discordance> discs = new HashSet<Discordance>();

		for (ExampleHolder ex : list) {
			Discordance disc = new Discordance();
			disc.setCallName(cleanCallName(ex.getCall())); //remove e.g. Both_rs1223
			disc.setDiscordant(discordance);
			disc.setFileName(ex.getFileName());
			disc.setSampleId(ex.getSampleName());
			disc.setWell(ex.getWell());

			discs.add(disc);
		}

		return discs;
	}
/**
 * convert string both_rs123457_AG -> AG or Undetermined_rs234234 -> ""
 * @param h
 * @return
 */
	private static String cleanCallName(String h) {
		String result = "";
		if (h.contains("Both_"))
			result= h.split("_")[2];
		else if (h.contains(UNDETERMINED)) 
			result="";
		else
			result=h;
		return result;
	}

	
	/**
	 * get amount of entries for a given CALL eg. (CT = 100, CC = 30, ...)
	 * @param call
	 * @param dataList
	 * @return
	 */
	private static int getNum(String call, LinkedList<ExampleHolder> dataList) {
		int result = 0;

		for(ExampleHolder ex : dataList){
			String sCall = cleanCallName(ex.getCall());
			String sCallrev = new StringBuffer(sCall).reverse().toString();
			if((sCall.equalsIgnoreCase(call) || sCallrev.equals(call)) && !sCall.equals(""))
				result++;
		}

		return result;
	}

	private static int getEqualExamples(LinkedList<ExampleHolder> dataList) {
		int sum = 0;
		HashSet<String> tmpSet = new HashSet<String>();
		HashSet<String> alreadyCount = new HashSet<String>();

		for(ExampleHolder ex : dataList){
			String sName = ex.getSampleName();
			String call = ex.getCall();
			
			String tmp = sName+call;
		
			if(tmpSet.contains(tmp)){
				sum++;
				alreadyCount.add(sName);
			} else
				tmpSet.add(tmp);
		}

		return sum;
	}

	private static void sortOutEqualExamples(LinkedList<ExampleHolder> dataList){
		LinkedList<ExampleHolder> deleteList = new LinkedList<ExampleHolder>();
		HashSet<String> tmpSet = new HashSet<String>();

		for(ExampleHolder ex : dataList){
			String tmp = ex.getSampleName()+ex.getCall();
			if(tmpSet.contains(tmp)) {
				deleteList.add(ex);
			} else {
				tmpSet.add(tmp);
			}
		}

		dataList.removeAll(deleteList);
	}

	// 7.4
	private static LinkedList<ExampleHolder> sortOutExamples(LinkedList<ExampleHolder> dataList){
		HashSet<String> tmpSet = new HashSet<String>();
		LinkedList<ExampleHolder> deleteList = new LinkedList<ExampleHolder>();
		LinkedList<ExampleHolder> emptyHelper = new LinkedList<ExampleHolder>();

		for(ExampleHolder ex : dataList) {
			String name = ex.getSampleName();
			String call = ex.getCall();

			if (tmpSet.contains(name)) {
				ExampleHolder tmp = getBySampleName(emptyHelper, name);
				if(call == null || call.isEmpty() || call.contains("Undetermined")) {
					deleteList.add(ex);
				} else if (tmp != null) {
					deleteList.add(tmp);
				}
			} else {
				if(call == null || call.isEmpty() || call.contains("Undetermined")) {
					emptyHelper.add(ex);
				}
				tmpSet.add(name);
			}
		}

		dataList.removeAll(deleteList);

		return deleteList;
	}

	private static ExampleHolder getBySampleName(LinkedList<ExampleHolder> list, String name) {
		for(ExampleHolder ex : list) {
	
			if (ex.getSampleName().equals(name)) {
				return ex;
			}
		}
		return null;
	}

	//7.3
	private static int getDiffExamples(LinkedList<ExampleHolder> dataList) {
		int sum = 0;
		HashSet<String> helpSet = new HashSet<String>();

		for(ExampleHolder ex : dataList) {
			String name = ex.getSampleName();
			String call = ex.getCall();

			if (call != null && !call.isEmpty() &&  !call.contains("Undetermined") && helpSet.contains(name)) {
				helpSet.add(name);
			
				sum++;
			} else if (call != null && !call.isEmpty() && !call.contains("Undetermined")) {
				helpSet.add(name);
			}
		}

		return sum;
	}

	//7.3
	private static LinkedList<ExampleHolder> sortOutDiffExamples(LinkedList<ExampleHolder> dataList){
		HashSet<String> helpSet = new HashSet<String>();
		LinkedList<ExampleHolder> diffDiscordantsAll = new LinkedList<ExampleHolder>();
		LinkedList<ExampleHolder> diffDiscordant1 = new LinkedList<ExampleHolder>();
ExampleHolder help=null;
		for(ExampleHolder ex : dataList) {
			String name = ex.getSampleName();
			
			String call = ex.getCall();
			if (call != null && !call.isEmpty() && !call.contains("Undetermined") && helpSet.contains(name)) {
				helpSet.add(name);
				diffDiscordantsAll.add(ex);
				diffDiscordantsAll.add(help);
				diffDiscordant1.add(ex);
			} else if (call != null && !call.isEmpty() && !call.contains("Undetermined")) {
				helpSet.add(name);
				help=ex;
				
			}
		}

		dataList.removeAll(diffDiscordant1);
	//	

		return diffDiscordantsAll;
	}

	private static void resetCalls(LinkedList<ExampleHolder> dataList, LinkedList<ExampleHolder> diff) {
		for (ExampleHolder ex : dataList) {
			for (ExampleHolder ex2: diff) {
				if (ex.getSampleName().equalsIgnoreCase(ex2.getSampleName())){
					ex.setCall("");
					break;
				}
			}
		}
	}

	public static String getSnpName(LinkedList<ExampleHolder> examples) {
		HashSet<String> names = getNames(examples);

if (names.size()==0){
	 names = getNamesUndet(examples);
}
		String snName = "";
		for (String name : names) {
	
			if (!name.isEmpty() && name.contains("_") && !name.contains("Undet")) {
				int lI = name.lastIndexOf("_");
				snName = name.substring(0, lI);
			}
			if (!name.isEmpty() && name.contains("Undetermined_")){
				snName=name.split("_")[1];
			}
		}
		return snName;
	}

	public static LinkedList<String[]> getDetectors(LinkedList<ExampleHolder> dataList, final Locale locale){
		ResourceBundle labels = ResourceBundle.getBundle("workflow", locale);

		LinkedList<String[]> overview = new LinkedList<String[]>();
		int undetCount = 0;
		int bothCount = 0;
		int h1Count = 0;
		int h2Count = 0;
		int emptyCount = 0;
		String h1Name="";
		String h2Name="";
		synchronized (dataList) {
		Object[] names = getNames(dataList).toArray();


		
		if (names.length>1){
		 h1Name = (String) names[0];
		 h2Name = (String) names[1];
		}
		
int hasBoth=0;		

	

for(ExampleHolder eh : dataList){
	//System.out.println("getDetect: + " + dataList.size() + eh.getSampleName() + " " + eh.getWell());
			String call = eh.getCall();
		
			if(call.contains(UNDETERMINED))
				++undetCount;
			else if (call.equals(BOTH)){			//for TaqMan
				++bothCount;
			}
			else if(call.contains(BOTH+"_")){		//for Sequenom
				++bothCount;
				if (hasBoth==0){
					String h= call.split("_")[2];
					String hholder=""+h.charAt(0);
					
					if (h1Name.length()==0 && h2Name.length() >0){
						String h2=h2Name.split("_")[1];
							if (hholder.equals(h2)){
							h1Name=h2Name.split("_")[0]+"_"+h.charAt(1);
						}
						else{
							h1Name=h2Name.split("_")[0]+"_"+h.charAt(0);
						}
					}
					else if  (h2Name.length()==0 && h1Name.length() >0){
						String h1=h1Name.split("_")[1];
					if (hholder.equals(h1)){	
						h2Name=h1Name.split("_")[0]+"_"+h.charAt(1);
					}
					else
						h2Name=h1Name.split("_")[0]+"_"+h.charAt(0);
			
					}
					else if (h1Name.length()==0 && h2Name.length()==0){
						h1Name=call.split("_")[1]+"_"+h.charAt(0);
						h2Name=call.split("_")[1]+"_"+h.charAt(1);
					}
					hasBoth=1; 
				}
			}
			else if(call.equalsIgnoreCase(h1Name))
				++h1Count;
			else if(call.equalsIgnoreCase(h2Name))
				++h2Count;
			else if(call.equalsIgnoreCase(""))
				++emptyCount;
	
}


}
System.out.println(" H1 " + h1Name  +  " H2 " + h2Name) ;
		String h1Val = getValue(h1Name);
		String h2Val = getValue(h2Name);

		String hom1[] = new String[3];
		hom1[0] = h1Name;
		hom1[1] = h1Val;
		hom1[2] = String.valueOf(h1Count);

		String hom2[] = new String[3];
		hom2[0] = h2Name;
		hom2[1] = h2Val;
		hom2[2] = String.valueOf(h2Count);

		String both[] = new String[3];
		both[0] = BOTH+"_"+h1Name;

		if (!h1Val.isEmpty() && !h2Val.isEmpty())
			both[1] = getSortedBoth(h1Val.substring(0, 1), h2Val.substring(0, 1));
		else
			both[2] = "";
		both[2] = String.valueOf(bothCount);

		String undet[] = new String[3];
		undet[0] = UNDETERMINED+"_"+h1Name;
		undet[1] = "";
		undet[2] = String.valueOf(undetCount);

		String empty[] = new String[3];
		empty[0] = labels.getString("lbEmptyWells")+"_"+h1Name;
		empty[1] = "";
		empty[2] = String.valueOf(emptyCount);

		overview.add(hom1);
		overview.add(hom2);
		overview.add(both);
		overview.add(undet);
		//overview.add(empty);   //REMOVED Absprache mit Margot und Stefan 11.04.2011

		return overview;
	}

	private static String getSortedBoth(String a, String b) {
		String[] sArr = {a, b};
		Arrays.sort(sArr);
		return sArr[0] + sArr[1];
	}

	private static HashSet<String> getNames(List<ExampleHolder> dataList){
		HashSet<String> names = new HashSet<String>();

		synchronized (dataList) {

			for(ExampleHolder eh : dataList){
		
			String call = eh.getCall();
		
			if(!call.contains(UNDETERMINED) && !call.contains(BOTH)
				&& !call.equalsIgnoreCase(NTC)
				&& !call.equalsIgnoreCase(ER)
				&& !call.isEmpty())
					names.add(call);
		}

		if(names.size() == 1) {
			names.add("");
		}
		}
		return names;
	}
	
		
	private static HashSet<String> getNamesUndet(LinkedList<ExampleHolder> dataList){
		HashSet<String> names = new HashSet<String>();

		for(ExampleHolder eh : dataList){
		
			String call = eh.getCall();
		
		 if (call.contains(UNDETERMINED)){
				names.add(call);
			}
		}

		if(names.size() == 1) {
			names.add("");
		}

		return names;
	}
	

	private static double calcMaf(int aa, int ab, int bb) {
		double maf = 0.0;

		double part1 = ab + 2*bb;
		double part2 = 2*(aa+ab+bb);
		maf = (part1 / part2) * 100;

		return maf;
	}

	private static Hwe calcHwe(final int homoObserved1, final int homoObserved2, final int heteroObserved) {
		Hwe hwe = new Hwe();

		hwe.setHomoObserved1(homoObserved1);
		hwe.setHomoObserved2(homoObserved2);
		hwe.setHeteroObserved(heteroObserved);

		int sumObserved = homoObserved1+homoObserved2+heteroObserved;
		Double alleleFreq = getAlleleFreq(homoObserved1, homoObserved2, heteroObserved);
		if (alleleFreq.isNaN())
			alleleFreq=0.0;
		
		Double homoExpected1 = getHomoExpected1(alleleFreq, sumObserved);
		if (homoExpected1.isNaN())
			homoExpected1=0.0;
		hwe.setHomoExpected1(homoExpected1);

		Double homoExpected2 = getHomoExpected2(alleleFreq, sumObserved);
		if (homoExpected2.isNaN())
			homoExpected2=0.0;
	
		hwe.setHomoExpected2(homoExpected2);

		Double heteroExpected = getHeteroExpected(alleleFreq, sumObserved);
		if (heteroExpected.isNaN())
			heteroExpected=0.0;
	
		hwe.setHeteroExpected(heteroExpected);

		double[] expected = {homoExpected1, heteroExpected, homoExpected2};
		long[] observed = {homoObserved1, heteroObserved, homoObserved2};
		
		Double chiSquare;
		if (sumObserved>0)
		   chiSquare = getChiSquare(expected, observed);
		else
			chiSquare=0.0;
		
			if (chiSquare.isNaN())
			hwe.setChiSquare(0.0);
		else
			hwe.setChiSquare(CalcUtils.calc_x_df(chiSquare, 1));

		double exact = getFisherExact(homoObserved1, homoObserved2, heteroObserved);
		hwe.setExact(exact);

		
		return hwe;
	}

	private static double getChiSquare(final double[] expected, final long[] observed) {
		Double chiSquare = 0d;

		try {
			chiSquare = TestUtils.chiSquare(expected, observed);
		} catch (Exception e) {//IllegalArgumentException, MathException
			logger.error("Detector empty", e);
		}
		if (!chiSquare.isNaN()) 
			return chiSquare;
			else
				return 0.0;
	}

	public static double getChiSquare1(final Snp snp, final double[] expected) {
		Double chiSquare = 0d;
		Hwe hwe = snp.getHwe();
		final long[] observed = {hwe.getHomoObserved1(), hwe.getHeteroObserved(), hwe.getHomoObserved2()};

		try {
			chiSquare = TestUtils.chiSquare(expected, observed);
		} catch (Exception e) {//IllegalArgumentException, MathException
			e.printStackTrace();
		}
		if (!chiSquare.isNaN()) 
		return chiSquare;
		else
			return 0.0;
	}

	private static double getFisherExact(final int homoObserved1, final int homoObserved2, final int heteroObserved) {
		double fisher = 0d;

		return fisher;
	}

	private static double getAlleleFreq(final int homoObserved1, final int homoObserved2, final int heteroObserved) {
		return (homoObserved2+(0.5*heteroObserved))/(homoObserved1+homoObserved2+heteroObserved);
	}

	private static double getHomoExpected1(final double alleleFreq, final int sumObserved) {
		return Math.pow(1-alleleFreq, 2)*sumObserved;
	}

	private static double getHomoExpected2(final double alleleFreq, final int sumObserved) {
		return Math.pow(alleleFreq, 2)*sumObserved;
	}

	private static double getHeteroExpected(final double alleleFreq, final int sumObserved) {
		return 2*alleleFreq*(1-alleleFreq)*sumObserved;
	}

	private static String getValue(String call){
		String letter = "";
		if(!call.isEmpty() && call.contains("_")) {
			int lI = call.lastIndexOf("_") + 1;
			letter = call.substring(lI, call.length());
			letter = letter + letter;
		}
		return letter;
	}
	
	@SuppressWarnings("unchecked")
	public static LinkedList<String>  getSexSnps(String snpid){
		String schemaname = InitServlet.dbSchemaname;
		String sql = "SELECT distinct genotype FROM "+schemaname+".GENOTYPE g WHERE g.snp_fk = "+snpid + " order by genotype desc";

		LinkedList<String> geno =  new LinkedList<String>(WorkflowDAO.getCurrentSession().createSQLQuery(sql).list());

		return geno;
	}
	
	
	@SuppressWarnings("unchecked")
	public static HashMap<String, String>  getGenotypes(String snpid){
		String schemaname = InitServlet.dbSchemaname;
	
		String sql = "select sample_id from "+schemaname+".GENOTYPE where snp_fk = "+snpid+" order by sample_id";
		String sql2 = "select genotype from "+schemaname+".GENOTYPE where snp_fk = "+snpid+" order by sample_id";
		
		LinkedList<String> id =  new LinkedList<String>(WorkflowDAO.getCurrentSession().createSQLQuery(sql).list());
		LinkedList<String> geno =  new LinkedList<String>(WorkflowDAO.getCurrentSession().createSQLQuery(sql2).list());

		HashMap<String, String> hm = new HashMap<String, String>();
		for (int i = 0; i<id.size(); i++){
			hm.put(id.get(i), geno.get(i));
		}

		return hm;
	}
	
	public static String getInteger(String snpId) {
		Pattern intsOnly = Pattern.compile("\\d+");
		Matcher makeMatch = intsOnly.matcher(snpId);
		makeMatch.find();
		String inputInt = makeMatch.group();
		return inputInt;
	}
	
}