package com.workflow.management;

import java.io.OutputStream;
import java.math.BigInteger;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.zip.ZipOutputStream;

import org.apache.log4j.Logger;
import org.hibernate.Session;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import com.workflow.database.DatabaseUtils;
import com.workflow.database.WorkflowDAO;
import com.workflow.database.beans.Sample;
import com.workflow.database.beans.Sexsnp;
import com.workflow.database.beans.Snp;
import com.workflow.database.beans.Study;
import com.workflow.servlet.InitServlet;

public class StudyManagement {
	private static final String CONCENTRATION2 = "concentration";
	private static final String SAMPLE_ID = "sampleId";
	private static final Logger logger = Logger.getLogger(StudyManagement.class);
	
	public static Study createStudy(Study study){
		int id = DatabaseUtils.add(study);
		Study inserted = DatabaseUtils.get(Study.class, id);

		return inserted;
	}

	public static void updateStudy(Study study){
		DatabaseUtils.update(study);
	}

	public static void activateStudy(Study study){
		study.setActive(true);
		DatabaseUtils.update(study);
	}

	public static void deactivateStudy(Study study){
		study.setActive(false);
		DatabaseUtils.update(study);
	}

	public static List<Study> getAllStudies(){
		return DatabaseUtils.getList(Study.class, "name");
	}

	public static void importSampleIds(List<String> sampleIds, int studyId){
		Study study = DatabaseUtils.get(Study.class, studyId);

		for(String id : sampleIds){
			Sample sample = new Sample();
			sample.setSampleId(id);
			sample.setStudy(study);
			sample.setIsError(false);
			sample.setInsertedWhen(new Date());

			try{
				DatabaseUtils.add(sample);
			} catch (Exception e){
				logger.error("Ignore constraint violation");
			}
		}
	}

	public static void setSamples(List<String> sampleIds, Study study){
		Set<Sample> samples = new HashSet<Sample>();
		for(String id : sampleIds){
			Sample sample = new Sample();
			sample.setSampleId(id);
			sample.setStudy(study);
			sample.setIsError(false);
			sample.setInsertedWhen(new Date());
			samples.add(sample);
		}
		study.setSamples(samples);
	}

	public static void importSamples(String studyId, HashMap<String, String[]> samples){
		Study study = DatabaseUtils.get(Study.class, Integer.parseInt(studyId));

		Set<Sample> studySamples = study.getSamples();
		List<Sample> updateSamples = new LinkedList<Sample>();
		for(Sample sample : studySamples){
			String sampleId = sample.getSampleId();
			if(samples.containsKey(sampleId)){
				String concentration = samples.get(sampleId)[0];
				String qualityValue = samples.get(sampleId)[1];

				sample.setConcentration(concentration);
				sample.setQualityValue(qualityValue);

				updateSamples.add(sample);
			}
		}

		DatabaseUtils.update(updateSamples);
	}

	@SuppressWarnings("unchecked")
	public static String getSampleIds(String studyId){
		String sql = "SELECT DISTINCT(s.sample_id) FROM SAMPLE s " +
		"WHERE s.study_fk = "+studyId+" ORDER BY CAST(sample_id AS SIGNED) ASC";

		LinkedList<String> sampleIds =  new LinkedList<String>(WorkflowDAO.getCurrentSession().createSQLQuery(sql).list());
		JSONObject jo = new JSONObject();
		JSONArray ja = new JSONArray();
		for(String sampleId: sampleIds){
			JSONObject tmp = new JSONObject();
			tmp.element(SAMPLE_ID, sampleId);
			ja.add(tmp);
		}
		jo.element("data", ja);

		return jo.toString();
	}

	public static Sample getSampleById(String id){
		return DatabaseUtils.get(Sample.class, Integer.parseInt(id));
	}

	public static String updateSample(Sample sample){
		String result = "{success: true, message: \"Sample updated succesfully\"}";

		try{
			DatabaseUtils.update(sample);
		}catch(Exception e){
			result = "{success: false, message: \"Failure during sample update\"}";
		}
		return result;
	}

	@SuppressWarnings("unchecked")
	public static JSONObject getSamplesFromStudyFetch(String studyId, String min, String max){
		String schemaname = InitServlet.dbSchemaname;
		System.out.println("SCHEMANAME +++++   " + schemaname);
		Session session = WorkflowDAO.getCurrentSession();
		String countQuery = "SELECT COUNT(*) FROM "+schemaname+".SAMPLE WHERE study_fk = " + studyId;
		String query = "SELECT {SAMPLE.*} FROM "+schemaname+".SAMPLE WHERE study_fk = " + studyId; //" ORDER BY CAST(CAST(sample_id AS BINARY) AS INT) ASC";

		BigInteger totalCount = (BigInteger) session.createSQLQuery(countQuery).uniqueResult();
		System.out.println("MIN"+min);
		System.out.println("MIN"+min);
		System.out.println("MAX"+studyId);
		List<Sample> samples = session.createSQLQuery(query).addEntity("SAMPLE", Sample.class)
			.setFirstResult(Integer.parseInt(min))
			.setMaxResults(Integer.parseInt(max))
			.list();

		JSONObject jo = new JSONObject();
		JSONArray ja = new JSONArray();
		for(Sample sample : samples){
			JSONObject tmp = new JSONObject();
			tmp.element("id", sample.getId());
			tmp.element(SAMPLE_ID, sample.getSampleId());

			tmp.element(CONCENTRATION2, sample.getConcentration());
			tmp.element("qualityValue", sample.getQualityValue());
			SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
			tmp.element("insertedWhen", sdf.format(sample.getInsertedWhen()));
			tmp.element("isError", sample.getIsError());
			ja.add(tmp);
		}
		logger.info("totalcount " + totalCount);
		jo.element("totalCount", totalCount);
		jo.element("data", ja);

		return jo;
	}

	public static JSONObject getStudy(String studyId){
		Study study = DatabaseUtils.get(Study.class, Integer.parseInt(studyId));
		JSONObject jo = new JSONObject();
		jo.element("numMultiple", study.getNumMultiple());
		jo.element("name", study.getName());
		jo.element("description", study.getDescription());
		jo.element("identifier", study.getIdentifier());
		jo.element("id", study.getId());
		return jo;
	}

	public static byte[] getReport(String idList){
		String ids[] = idList.split("_");

		List<Sample> samples = getSamplesFromList(ids);

		StringBuffer line = new StringBuffer();
		line.append("\"SampleID\"");line.append(';');
		line.append("\"DNA Konzentration [ng/l]\"");line.append(';');
		line.append("\"Ratio A260/280\"");line.append(';');
		line.append("\"Layout Positionen\"");line.append(';');
		line.append("\"Is Error\"");line.append(';');line.append('\n');

		for(Sample sample : samples){
			line.append("\""+sample.getSampleId()+"\"");line.append(';');
			line.append("\""+sample.getConcentration()+"\"");line.append(';');
			line.append("\""+sample.getQualityValue()+"\"");line.append(';');

			String isError = (sample.getIsError()) ? "Ja" : "Nein";
			line.append("\""+isError+"\"");line.append(';');line.append('\n');
		}

		return line.toString().getBytes();
	}

	@SuppressWarnings("unchecked")
	private static List<Sample> getSamplesFromList(String[] ids){
		String schemaname = InitServlet.dbSchemaname;
		String i = "";
		for(String id : ids){
			id = id.trim();
			if(id.length() > 0)
				i += id + ",";
		}
		i = i.substring(0, i.length()-1);
		String query = "SELECT {SAMPLE.*} FROM "+schemaname+".SAMPLE WHERE id IN ("+i+")";

		Session session = WorkflowDAO.getCurrentSession();

		return (List<Sample>) session.createSQLQuery(query).addEntity("SAMPLE", Sample.class).list();
	}

	public static Study getStudyById(String studyId){
		return DatabaseUtils.get(Study.class, Integer.parseInt(studyId));
	}

	
	public static int importSexSnp(List<Sexsnp> failures, String snpId){
		System.out.println("Import " + snpId);
		Snp snp = DatabaseUtils.get(Snp.class, Integer.valueOf(snpId));
		snp.setSex(1);
		System.out.println("snip: " + snp.getName() + " : " + snp.getSexsnp() + " failure.size " + failures.size() + " sex " + snp.getSex());
				
		
		for(Sexsnp id : failures){
			try{
				System.out.println("id " + id.getSampleId() + " " + id.getGenotype() +  " " + id.getPhenotype() +  " " + id.getSampleId() + " " );
				id.setSnp(snp);
				DatabaseUtils.add(id);
			} catch (Exception e){
				e.printStackTrace();
				logger.error("Ignore constraint violation");
				return 0; //error
			}
		}
	
		DatabaseUtils.update(snp);
		return 1; //ok

	}
	
	
}