package com.workflow.servlet;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Locale;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

import com.workflow.database.beans.Snp;
import com.workflow.database.beans.Study;
import com.workflow.management.SNPManagement;
import com.workflow.management.StudyManagement;

@SuppressWarnings("serial")
public class DownloadServlet extends HttpServlet {

    private static final String ATTACHMENT_FILENAME = "attachment;filename=";
	private static final String APPLICATION_ZIP = "application/zip";
	private static final String APPLICATION_VND_MS_EXCEL = "application/vnd.ms-excel";
	private static final String TEXT_CSV = "text/csv";
	private static final String CONTENT_DISPOSITION = "Content-Disposition";
	private static final Logger logger = Logger.getLogger(DownloadServlet.class);
	public DownloadServlet() {
        super();
    }

	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		processRequest(request, response);
	}

	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		processRequest(request, response);
	}

	private void processRequest(HttpServletRequest request, HttpServletResponse response) throws IOException{
		String downloadType = request.getParameter("downloadType");
		final Locale locale = java.util.Locale.US; 
		getServletContext().getMimeType("");

			byte result[] = null;
		if(downloadType.equals("report")){
			String idList = request.getParameter("sampleIds");
			String studyId = request.getParameter("studyId");
			Study study = StudyManagement.getStudyById(studyId);
			String contentDisp = ATTACHMENT_FILENAME
				+study.getIdentifier()+"_"
				+study.getName()+"_report.csv";

			result = StudyManagement.getReport(idList);

			response.setContentType(TEXT_CSV);
			response.setHeader(CONTENT_DISPOSITION, contentDisp);

			OutputStream out = null;
			try {
				out = response.getOutputStream();
				out.write(result);
			} catch (IOException e) {
				e.printStackTrace();
			} finally {
				if(out != null)
					out.close();
			}
		}  if(downloadType.equals("genotypes")){
			String snpId = request.getParameter("snpId");
			boolean genotypes = request.getParameter("genotypes") != null;
			boolean summary = request.getParameter("summary") != null;
			boolean excel = request.getParameter("excel") != null;
			boolean hapmap = request.getParameter("hapmap") != null;
			
					
			boolean[] checkHapMap = new boolean[16];
			for (int i =1; i<16; i++){
				checkHapMap[i-1]= request.getParameter("cb"+i) != null;
			}
			checkHapMap[15]= request.getParameter("cb"+16) != null;
			
			String exportName = request.getParameter("export");
			boolean zipDownload = (excel && summary) || (excel && genotypes) || (excel && hapmap) || (summary && genotypes) || (summary && hapmap) || (genotypes && hapmap);
			String extension = ".tsv";
			String contentType = TEXT_CSV;
			String suffix = "";
			String multi = request.getParameter("multi");
			
			Snp snp=null;
			Vector<String> snpvector = new Vector<String>();
			if (multi.equals("null"))
			  {snp= SNPManagement.getSnpById(snpId);}
			else{
				int i=0;
				 StringTokenizer st = new StringTokenizer(snpId);
			     while (st.hasMoreTokens()) {
			    	 String value = st.nextToken();
			    	
			    	if (!value.equals("undefined")){
			    		try{
			    			i = Integer.valueOf(SNPManagement.getInteger(snpId));
			    			snpvector.add(value);
			    		}catch (Exception e) {
			    			 logger.error("wrong SNPID value " + value);
							}
			    		}
			    	else{
			    		logger.error("wrong SNPID value " + value);
			    	}
			    	
			     }
			     try{
				snp= SNPManagement.getSnpById(snpvector.get(0));
			     }
			     catch (Exception e) {
					snp=null;
				}
			     }
			if (snp!=null){
			if(zipDownload) {
				response.setContentType(APPLICATION_ZIP);
				response.setHeader(CONTENT_DISPOSITION, ATTACHMENT_FILENAME
						+snp.getName() + ".zip");
				SNPManagement.getAllZipDownload(response.getOutputStream(), snp, excel, genotypes, summary, hapmap, locale);
			} else if (excel) {
				if (multi.equals("null")){
					response.setContentType(APPLICATION_VND_MS_EXCEL);
					response.setHeader(CONTENT_DISPOSITION, ATTACHMENT_FILENAME
							+snp.getName() + ".xls");
					SNPManagement.writeExcelSummary(snp, response.getOutputStream(), locale);
				}
				else{
				response.setContentType(APPLICATION_ZIP);
			    response.setHeader(CONTENT_DISPOSITION, ATTACHMENT_FILENAME
					+exportName + ".zip");
				ZipOutputStream zipStream = new ZipOutputStream(response.getOutputStream());
				
				for ( int i =0; i <snpvector.size(); i++){
					ByteArrayOutputStream baos = new ByteArrayOutputStream();

					snp = SNPManagement.getSnpById(snpvector.get(i));
					SNPManagement.writeExcelSummary(snp, baos, locale);
					try{
					zipStream.putNextEntry(new ZipEntry(snp.getName()+"_"+ snp.getDirectory() + ".xls"));
					zipStream.write(baos.toByteArray());
					}catch (Exception e) {
						zipStream.putNextEntry(new ZipEntry(snp.getName()+"_"+ snp.getDirectory()+"_"+snp.getId() + ".xls"));
						zipStream.write(baos.toByteArray());
					
						e.printStackTrace();
					}
				}
				ByteArrayOutputStream baos = new ByteArrayOutputStream();
				SNPManagement.writeExcelGenotypesOverview(snpvector, baos, locale);
				zipStream.putNextEntry(new ZipEntry(exportName+"_genotypesOverview"+ ".xls"));
				zipStream.write(baos.toByteArray());
			
				ByteArrayOutputStream baos1 = new ByteArrayOutputStream();

				SNPManagement.writeExcelSummaryTotal(snpvector, baos1, locale, checkHapMap);
				
				zipStream.putNextEntry(new ZipEntry(exportName+"_summaryTotal"+ ".xls"));
				zipStream.write(baos1.toByteArray()); 

				zipStream.closeEntry();
				try {
					zipStream.close();
				} catch (IOException e) {
					e.printStackTrace();
					}
				}
			} else if (genotypes) {
				result = SNPManagement.getGenotypeDownload(snp, locale);
			} else if (hapmap) {
				result = SNPManagement.getHapmapDownload(snp, locale);
				suffix = "_HapMap";
			} else if (summary) {
				result = SNPManagement.getSummaryDownload(snp, locale);
				suffix = "_QC";
			}
			}
			OutputStream out = null;
			try {
				if (!excel && !zipDownload) {
					String contentDisp = ATTACHMENT_FILENAME
						+snp.getName() + suffix + extension;

					response.setContentType(contentType);
					response.setHeader(CONTENT_DISPOSITION, contentDisp);

					out = response.getOutputStream();
					out.write(result);
				}
			} catch (IOException e) {
				e.printStackTrace();
			} finally {
				if(out != null && !excel)
					out.close();
			}
		} if(downloadType.equals("roboter-csv")){
			String studyId = request.getParameter("studyId");
			String layoutNr = request.getParameter("layoutNr");
			String type = request.getParameter("type");
			Study study = StudyManagement.getStudyById(studyId);

			if(type.equals("zip")){
				String contentDisp = "attachment;filename=\""
					+study.getIdentifier()+"_"
					+study.getName()+".zip\"";

				response.setContentType("application/octet-stream");
				response.setHeader(CONTENT_DISPOSITION, contentDisp);

				try {
//					StudyManagement.getZipForRoboter(study, srcLabel, response.getOutputStream());
				} catch (Exception e) {}
			} else {
				String contentDisp = ATTACHMENT_FILENAME
					+study.getIdentifier()+"_"
					+study.getName()+"_"+layoutNr+".csv";

//				result = StudyManagement.getRoboterFromLayout(study, layoutNr, srcLabel);

				response.setContentType(TEXT_CSV);
				response.setHeader(CONTENT_DISPOSITION, contentDisp);

				OutputStream out = null;
				try {
					out = response.getOutputStream();
					out.write(result);
				} catch (IOException e) {
					e.printStackTrace();
				} finally {
					if(out != null)
						out.close();
				}
			}
		}
	}
}
