package com.workflow.servlet;

import java.io.IOException;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Set;
import java.util.Map.Entry;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import com.workflow.database.beans.Discordance;
import com.workflow.database.beans.Hapmap;
import com.workflow.database.beans.Hwe;
import com.workflow.database.beans.Sexsnp;
import com.workflow.database.beans.Snp;
import com.workflow.database.beans.SnpContainer;
import com.workflow.management.HapMapManagement;
import com.workflow.management.SNPManagement;
import com.workflow.management.StudyManagement;
import com.workflow.management.Thread1000G;
import com.workflow.management.beans.ExampleHolder;
import com.workflow.management.beans.SexSnpHolder;

@SuppressWarnings("serial")
public class SNPDataServlet extends AbstractServlet {
	private static final Logger logger = Logger.getLogger(SNPDataServlet.class);
	private static final String NEW_DATA_LIST = "newDataList";
	private LinkedList<SnpContainer> snpcList;
	int start = 0;

	private static enum RequestType {
		getDeleteOverview("getDeleteOverview"), calculate("calculate"), calculateSexSnp(
				"calculateSexSnp"), confirmDelete("confirmDelete"), getSnp(
				"getSnp"), getCurrentSnp("getCurrentSnp"), getDetectorNames(
				"getDetectorNames"), getSex("getSex"), deleteSNP("deleteSNP"), removeSNPfromHashList(
				"removeSNPfromHashList");

		private final String requestType;

		RequestType(String requestType) {
			this.requestType = requestType;
		}

		private String getRequestType() {
			return requestType;
		}

		public static RequestType byValue(String requestTypeValue) {
			for (RequestType requestType : RequestType.values())
				if (requestType.getRequestType().equals(requestTypeValue))
					return requestType;

			throw new IllegalArgumentException("RequestType \""
					+ requestTypeValue + "\" is not supported");
		}
	};

	public SNPDataServlet() {
		super();
	}

	protected void doGet(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		processRequest(request, response);
	}

	protected void doPost(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		try {
			processRequest(request, response);
		} catch (Exception e) {
			logger.error("java.util.ConcurrentModificationException");
			e.printStackTrace();
		}
	}

	@SuppressWarnings("unchecked")
	private void processRequest(HttpServletRequest request,
			HttpServletResponse response) {
		RequestType requestType = getRequestType(request);
		final Locale locale = request.getLocale();

		if (requestType.equals(RequestType.getSnp)) {
			String snpId = request.getParameter("snpId");

			Snp snp = SNPManagement.getSnp(snpId);
			JSONObject jo = SnpToJson(snp);

			JSONObject result = new JSONObject();
			result.element("success", true);
			result.element("data", jo);

			setResponse(response, result.toString());
		} else if (requestType.equals(RequestType.getCurrentSnp)) {
			Snp snp = (Snp) request.getSession(false).getAttribute("result");
			JSONObject jo = SnpToJson(snp);

			JSONObject result = new JSONObject();
			result.element("success", true);
			result.element("data", jo);

			setResponse(response, result.toString());
		} else if (requestType.equals(RequestType.calculate)) {
			logger.debug("calculate " + snpcList.size());

			int snpId = 0;
		
			Thread1000G[] threads = new Thread1000G[snpcList.size()];
		
			   
			for (int i = 0; i < snpcList.size(); i++) {
				System.out.println("SNPC size " + snpcList.size());
				snpId = SNPManagement.writeSnpToDB(snpcList.get(i));

				Snp snp = SNPManagement.getSnp(String.valueOf(snpId));
				try {
					HapMapManagement.writeHapMap(snp);
					System.out.println("HAAAAPMAP drin - fehlt nur noch 1000G");
					//t1000g.add(new Thread1000G(snp).start());
					 threads[i] = new Thread1000G(snp);
					 threads[i].start();
				} catch (Exception e) {
					logger.error("Could not create HapMap:: ", e);
				}
				
				request.getSession(false).removeAttribute(NEW_DATA_LIST);
			}
		//Join Threads = wait for threads
			for (int i = 0; i < threads.length; i++) {
			    try {
			       threads[i].join();
			    } catch (InterruptedException ignore) {}
			}
			
			JSONObject result = new JSONObject();
			result.element("success", true);
			result.element("snpId", snpId);
			setResponse(response, result.toString());
		}

		else if (requestType.equals(RequestType.calculateSexSnp)) {

			System.out.println("here we go!");
			// logger.debug("calculateSexSnp " + snpcList.size());

			String snpIdIt = request.getParameter("snpId");
			String snpId = getInteger(snpIdIt);
			Snp snp = SNPManagement.getSnp(String.valueOf(snpId));

			String maleG = request.getParameter("maleG");
			String maleP = request.getParameter("maleP");
			String femaleG = request.getParameter("femaleG");
			String femaleP = request.getParameter("femaleP");

			List<SexSnpHolder> phenoList = SexSnpDataUploadServlet.dataList
					.get("key");

			HashMap<String, String> genoList = SNPManagement
					.getGenotypes(snpId);
			List<Sexsnp> failure = new LinkedList<Sexsnp>();

			for (int i = 0; i < phenoList.size(); i++) {
				if (genoList.get(phenoList.get(i).getSampleName()).length() > 0) {
					if (phenoList.get(i).getPhenotype().equals(maleP)) {
						if (!genoList.get(phenoList.get(i).getSampleName())
								.toUpperCase().equals(maleG.toUpperCase())) {

							Sexsnp s = new Sexsnp();
							s.setGenotype(genoList.get(phenoList.get(i)
									.getSampleName()) + " (female)");
							s.setPhenotype(phenoList.get(i).getPhenotype()
									+ " (male)");
							s.setSampleId(phenoList.get(i).getSampleName());
							s.setSnp(snp);
							failure.add(s);
						}
					}

					if (phenoList.get(i).getPhenotype().equals(femaleP)) {
						System.out.println(" IF Pheno = FEMALEP " + femaleP
								+ " " + femaleG);
						if (!genoList.get(phenoList.get(i).getSampleName())
								.toUpperCase().equals(femaleG.toUpperCase())) {
							Sexsnp s = new Sexsnp();
							s.setGenotype(genoList.get(phenoList.get(i)
									.getSampleName()) + " (male)");
							s.setPhenotype(phenoList.get(i).getPhenotype()
									+ " (female)");
							s.setSampleId(phenoList.get(i).getSampleName());
							s.setSnp(SNPManagement.getSnp(snpId));
							failure.add(s);
						}
					}
				}
			}

			int seximport = 0;
			if (failure.size() < (phenoList.size() / 2)) {
				seximport = StudyManagement.importSexSnp(failure, snpId);
			} else
				seximport = 0;

			// request.getSession(false).removeAttribute(NEW_DATA_LIST);
			if (seximport == 1) {
				JSONObject result = new JSONObject();
				result.element("success", true);
				result.element("snpId", getInteger(snpIdIt));
				setResponse(response, result.toString());
			} else {
				JSONObject result = new JSONObject();
				result.element("success", false);
				result.element("snpId", getInteger(snpIdIt));
				setResponse(response, result.toString());
			}
		}

		else if (requestType.equals(RequestType.removeSNPfromHashList)) {

			String studyId = request.getParameter("studyId");
			String snpName = request.getParameter("snpName");
			String hapmapName = request.getParameter("hapmapName");
			String snpComment = request.getParameter("snpComment");
			String snpInsertedBy = request.getParameter("snpInsertedBy");
			String replacements = request.getParameter("replacements");

			logger.info("REMOVE from Project" + studyId + "  " + snpName + " ");

			HashMap<String, String> repMap = getRepMap(JSONArray
					.fromObject(replacements));
			ConcurrentHashMap<String, List<ExampleHolder>> dataListHash = (ConcurrentHashMap<String, List<ExampleHolder>>) request
					.getSession(false).getAttribute(NEW_DATA_LIST);
			String snpFolder = (String) request.getSession(false).getAttribute(
					"labelfolder");
			LinkedList<ExampleHolder> dataList = (LinkedList<ExampleHolder>) dataListHash
					.get(dataListHash.keySet().toArray()[start]);
			SNPManagement.mergeList(dataList, studyId);

			// Collection dataList= (Collection<ExampleHolder>)
			// Collections.synchronizedList(dataList);
			SNPManagement.replaceGenoTypes(dataList, repMap);

			// OLD int snpId = SNPManagement.calculate(dataList, studyId,
			// snpName, snpComment, snpInsertedBy, repMap, hapmapName);

			// SOLVED BY PAUSE IN FRONTEND
			snpcList.add(SNPManagement.calculate(dataList, studyId, snpName,
					snpComment, snpInsertedBy, snpFolder, repMap, hapmapName));
			System.out.println("START " + start);

			dataListHash.remove(dataListHash.keySet().toArray()[start]);

			// for debugging only!!! simulates slow connection.
			/*try {
				System.out.print("Waiting......");
				Thread.sleep(2000);
				System.out.println("OK.");
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}*/
		}

		else if (requestType.equals(RequestType.confirmDelete)) {

			String sampleIds = request.getParameter("sampleIds");
			JSONArray data = JSONArray.fromObject(sampleIds);
			ConcurrentHashMap<String, List<ExampleHolder>> dataListHash = (ConcurrentHashMap<String, List<ExampleHolder>>) request
					.getSession(false).getAttribute(NEW_DATA_LIST);
			logger.info("deeleeete " + dataListHash.size() + "  "
					+ dataListHash.keySet());
			System.out.println("deeleeete " + dataListHash.size() + "  "
					+ dataListHash.keySet());
			// remove all NTC or ER from all SNPs
			for (int i = 0; i < dataListHash.size(); i++) {
				LinkedList<ExampleHolder> dataList = (LinkedList<ExampleHolder>) dataListHash
						.get(dataListHash.keySet().toArray()[i]);
				SNPManagement.deleteSampleIds(dataList, jsonToList(data));
			}
			// request.getSession(false).setAttribute(NEW_DATA_LIST, dataList);

			JSONObject result = new JSONObject();
			result.element("success", true);
			snpcList = new LinkedList<SnpContainer>();
			System.out.println(snpcList.size());
			setResponse(response, result.toString());
		} else if (requestType.equals(RequestType.getDeleteOverview)) {
			int start = 0;
			String studyId = request.getParameter("studyId");
			ConcurrentHashMap<String, List<ExampleHolder>> dataListHash = (ConcurrentHashMap<String, List<ExampleHolder>>) request
					.getSession(false).getAttribute(NEW_DATA_LIST);
			LinkedList<ExampleHolder> dataList = (LinkedList<ExampleHolder>) dataListHash
					.get(dataListHash.keySet().toArray()[start]);
			LinkedList<String[]> deleteOverview = SNPManagement
					.getDeleteOverview(dataList, studyId);
			SNPManagement.mergeList(dataList, studyId);

			JSONObject jo = new JSONObject();
			JSONArray ja = new JSONArray();
			for (String entry[] : deleteOverview) {
				JSONObject tmp = new JSONObject();
				tmp.element("name", entry[0]);
				tmp.element("nums", entry[1]);
				ja.add(tmp);
			}
			jo.element("data", ja);
			setResponse(response, jo.toString());
		}

		else if (requestType.equals(RequestType.getDetectorNames)) {

			if (request.getParameter("start") != null)
				start = Integer.valueOf(request.getParameter("start"));
			else
				start = 0;

			System.out.println("Detector + " + start);

			ConcurrentHashMap<String, List<ExampleHolder>> dataListHash = (ConcurrentHashMap<String, List<ExampleHolder>>) request
					.getSession(false).getAttribute(NEW_DATA_LIST);

			JSONObject jo = new JSONObject();
			JSONArray ja = new JSONArray();
			for (int i = 0; i < dataListHash.size(); i++) {
				if (i == start) {

					LinkedList<String[]> detectors = SNPManagement
							.getDetectors(
									(LinkedList<ExampleHolder>) dataListHash
											.get(dataListHash.keySet()
													.toArray()[i]), locale);

					// String snpName =
					// SNPManagement.getSnpName((LinkedList<ExampleHolder>)dataListHash.get(dataListHash.keySet().toArray()[i]));

					// logger.debug("snpNAME " +snpName + " detectors  " +
					// detectors.size());
					for (String entry[] : detectors) {
						JSONObject tmp = new JSONObject();
						String h = "" + entry[0];
						if (h.contains("Both"))
							h = "Both";
						if (h.contains("Undetermined"))
							h = "Undetermined";
						if (h.contains("Wells"))
							h = entry[0].split("_")[0];

						tmp.element("name", h);
						tmp.element("value", entry[1]);
						tmp.element("nums", entry[2]);
						tmp.element("snpName", h.split("_")[0]);
						tmp.element("isOk", 0);

						ja.add(tmp);
					}

					jo.element("data", ja);
				}
			}
			jo.element("totalCount", dataListHash.size());
			setResponse(response, jo.toString());
		} else if (requestType.equals(RequestType.getSex)) {
			JSONObject jo = new JSONObject();
			JSONArray ja = new JSONArray();
			logger.debug("getSex ");
			System.out.println("getsex");
			String snpId = request.getParameter("snpId");
			System.out.println(getInteger(snpId) + " - " + snpId);
			LinkedList<String> geno = SNPManagement
					.getSexSnps(getInteger(snpId));
			System.out.println(geno.get(0) + ": " + geno.get(1) + ": "
					+ geno.get(2) + " ");
			if (geno.get(2).length() == 0) // 2 snps found
			{
				System.out.println("GENO LONGER");
				JSONObject tmp = new JSONObject();
				System.out.println(":-: " + SexSnpDataUploadServlet.firstSex);
				tmp.element("genotype", geno.get(0).toUpperCase());
				tmp.element("phenotype", SexSnpDataUploadServlet.firstSex);
				ja.add(tmp);
				tmp.element("genotype", geno.get(1).toUpperCase());
				tmp.element("phenotype", SexSnpDataUploadServlet.secondSex);
				ja.add(tmp);
				jo.element("data", ja);
			} else { // 3 SNPs found
				JSONObject tmp = new JSONObject();
				tmp.element("genotype", geno.get(0).toUpperCase());
				ja.add(tmp);
				tmp.element("genotype", geno.get(1).toUpperCase());
				ja.add(tmp);
				tmp.element("genotype", geno.get(2).toUpperCase());
				ja.add(tmp);
				jo.element("data", ja);
			}
			setResponse(response, jo.toString());
		} else if (requestType.equals(RequestType.deleteSNP)) {
			String snpId = request.getParameter("snpId");
			Snp snp = null;

			StringTokenizer st = new StringTokenizer(snpId);

			while (st.hasMoreTokens()) {

				snp = SNPManagement.getSnpById(st.nextToken());
				SNPManagement.deleteSNP(snp);
			}

			JSONObject jo = new JSONObject();
			jo.element("success", true);
			jo.element("message", "lbDeleteSnpSuccess");

			setResponse(response, jo.toString());
		} else
			throw new IllegalArgumentException("Request Type \"" + requestType
					+ "\" is not supported");
	}

	private String getInteger(String snpId) {
		Pattern intsOnly = Pattern.compile("\\d+");
		Matcher makeMatch = intsOnly.matcher(snpId);
		makeMatch.find();
		String inputInt = makeMatch.group();
		return inputInt;
	}

	private JSONObject SnpToJson(Snp snp) {
		java.util.Locale.setDefault(java.util.Locale.US);
		JSONObject snpJson = new JSONObject();
		DecimalFormat f = new DecimalFormat("0.##E0");
		DecimalFormat f2 = new DecimalFormat("#0.00");
		DecimalFormat f3 = new DecimalFormat("#0.000");
		DecimalFormat f4 = new DecimalFormat("#0.0000");
		SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy");

		snpJson.element("createdWhen", sdf.format(snp.getCreatedWhen()));
		snpJson.element("name", snp.getName());
		snpJson.element("insertedBy", snp.getInsertedBy());
		snpJson.element("comment", snp.getComment());
		snpJson.element("nameHetero", snp.getNameHetero());
		snpJson.element("nameHomo1", snp.getNameHomo1());
		snpJson.element("nameHomo2", snp.getNameHomo2());
		snpJson.element("numDisc", snp.getNumDisc());
		snpJson.element("sex", snp.getSex());
		snpJson.element("numMultiple", snp.getNumMultiple());
		snpJson.element("numSamples", snp.getNumSamples());
		snpJson.element("procCallRate", f2.format(snp.getProcCallRate()) + "%");
		snpJson.element("procDiscRate", f2.format(snp.getProcDiscRate()) + "%");
		snpJson.element("procMaf", f2.format(snp.getProcMaf()) + "%");
		snpJson.element("procQc", f2.format(snp.getProcQc()));
		snpJson.element("procQcSamples", f2.format(snp.getProcQcSamples())
				+ "%");

		Hwe hwe = snp.getHwe();
		JSONObject hweJson = new JSONObject();
		hweJson.element("heteroExpected", f2.format(hwe.getHeteroExpected()));
		hweJson.element("heteroObserved", hwe.getHeteroObserved());
		hweJson.element("homoExpected1", f2.format(hwe.getHomoExpected1()));
		hweJson.element("homoObserved1", hwe.getHomoObserved1());
		hweJson.element("homoExpected2", f2.format(hwe.getHomoExpected2()));
		hweJson.element("homoObserved2", hwe.getHomoObserved2());
		hweJson.element("chiSquare", f.format(hwe.getChiSquare()));
		// logger.debug("formatted::" + String.format(Locale.GERMAN, "%e",
		// hwe.getChiSquare()));
		// logger.debug("pure::" + hwe.getChiSquare());
		hweJson.element("exact", f4.format(hwe.getExact()));

		snpJson.element("hwe", hweJson);
		ArrayList<String> help = new ArrayList<String>();
		JSONArray discs = new JSONArray();
		JSONArray discsInt = new JSONArray();
		int discTrue = 0, discFalse = 0;
		for (Discordance disc : snp.getDiscordances()) {
			if (disc.isDiscordant()) {
				discTrue++;
			} else if (!help.contains(disc.getSampleId())) {
				help.add(disc.getSampleId());
				discFalse++;
			}
		}

		for (Discordance disc : snp.getDiscordances()) {
			JSONArray ja = new JSONArray();
			ja.add(disc.getSampleId());
			ja.add(disc.getCallName());
			ja.add(disc.getWell());
			ja.add(disc.getFileName());
			ja.add(disc.isDiscordant());
			if (disc.isDiscordant())
				ja.add(discTrue);
			else
				ja.add(discFalse);

			discs.add(ja);
		}

		discsInt.add(discFalse);
		discsInt.add(discTrue);

		snpJson.element("discordances", discs);

		// ADD SEXSNP INFORMATION IF EXISTING

		JSONArray sex = new JSONArray();

		for (Sexsnp sx : snp.getSexsnp()) {
			JSONArray ja = new JSONArray();
			ja.add(sx.getSampleId());
			ja.add(sx.getGenotype());
			ja.add(sx.getPhenotype());
			sex.add(ja);
		}

		snpJson.element("sexsnp", sex);

		// ADD HAPMAP data

		JSONArray haps = new JSONArray();
		JSONArray ibk = new JSONArray();

		ibk.add("OWN DATA");

		ibk.add(snp.getNumHomo1());
		ibk.add(f3.format((double) snp.getNumHomo1()
				/ (double) snp.getNumSamples()));
		ibk.add(snp.getNumHomo2());
		ibk.add(f3.format((double) snp.getNumHomo2()
				/ (double) snp.getNumSamples()));
		ibk.add(snp.getNumHetero());
		ibk.add(f3.format((double) snp.getNumHetero()
				/ (double) snp.getNumSamples()));
		ibk.add(snp.getNumSamples());
		ibk.add("");

		haps.add(ibk);

		if (snp.getHapmaps() != null) {
			List<Hapmap> sorted = new LinkedList<Hapmap>(snp.getHapmaps());
			Collections.sort(sorted, new Comparator<Hapmap>() {
				public int compare(Hapmap h1, Hapmap h2) {
					return h1.getPopulation().compareToIgnoreCase(
							h2.getPopulation());
				}
			});

			for (Hapmap hapmap : sorted) {
				JSONArray hm = new JSONArray();

				hm.add(hapmap.getPopulation());
				hm.add(hapmap.getHomo1());
				hm.add(hapmap.getHomoFreq1());
				hm.add(hapmap.getHomo2());
				hm.add(hapmap.getHomoFreq2());
				hm.add(hapmap.getHetero());
				hm.add(hapmap.getHeteroFreq());
				hm.add(hapmap.getTotal());
				if (hapmap.getChi() == 0.0) {
					hm.add("<2.06E-9");
				} else {
					hm.add(f.format(hapmap.getChi()));
				}

				haps.add(hm);
			}
		}

		snpJson.element("hapmaps", haps);

		return snpJson;
	}

	private HashMap<String, String> getRepMap(JSONArray reps) {
		HashMap<String, String> map = new HashMap<String, String>();

		for (int i = 0, l = reps.size(); i < l; ++i) {
			JSONObject obj = reps.getJSONObject(i);
			map.put(obj.getString("key"), obj.getString("value"));
		}

		return map;
	}

	private LinkedList<String> jsonToList(JSONArray jA) {
		LinkedList<String> list = new LinkedList<String>();

		for (int i = 0, l = jA.size(); i < l; ++i) {
			JSONObject tmp = jA.getJSONObject(i);
			list.add(tmp.getString("name"));
		}
		return list;
	}

	private RequestType getRequestType(HttpServletRequest request) {
		String requestType = request.getParameter("requestType");
		return RequestType.byValue(requestType);
	}
}
