package com.workflow.servlet;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.fileupload.FileItemIterator;
import org.apache.commons.fileupload.FileItemStream;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.fileupload.util.Streams;
import org.apache.log4j.Logger;

import com.workflow.database.beans.Sample;
import com.workflow.database.beans.Snp;
import com.workflow.database.beans.Study;
import com.workflow.management.ProbeImporter;
import com.workflow.management.StudyManagement;

@SuppressWarnings("serial")
public class WorkflowServlet extends AbstractServlet {
	private static final Logger logger = Logger.getLogger(WorkflowServlet.class);

	private static enum RequestType {
		insertStudy("insertStudy"), insertStudyMin("insertStudyMin"),
		getAllStudies("getAllStudies"), getSamplesFromStudy("getSamplesFromStudy"),
		updateSample("updateSample"), getSampleIds("getSampleIds"),
		activateStudy("activateStudy"), deactivateStudy("deactivateStudy"),
		getStatistics("getStatistics"), getStudy("getStudy"), editStudy("editStudy");

		private final String requestType;

		RequestType(String requestType) {
			this.requestType = requestType;
		}

		private String getRequestType() {
			return requestType;
		}

		public static RequestType byValue(String requestTypeValue) {
			for (RequestType requestType : RequestType.values())
				if (requestType.getRequestType().equals(requestTypeValue))
					return requestType;

			throw new IllegalArgumentException("RequestType \"" + requestTypeValue + "\" is not supported");
		}
	};

    public WorkflowServlet() {
        super();
    }

	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		processRequest(request, response);
	}

	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		processRequest(request, response);
	}

	private void processRequest(HttpServletRequest request, HttpServletResponse response){
		boolean isMultipart = ServletFileUpload.isMultipartContent(request);
		if(isMultipart){
			try {
				handleFileUpload(request);
				response.setContentType("text/html");
				response.getWriter().write("<html><body><textarea>{\"response\":{\"message\":\"\"},\"success\":true}</textarea></body></html>");
			} catch (FileUploadException e) {
				System.out.println("!");
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
				System.out.println("!!");
			} catch (Exception e) {
		
				response.setContentType("text/html");
				try {
					response.getWriter()
							.write("<html><body><textarea>{\"response\":{\"message\":\"\"},\"success\":false}</textarea></body></html>");
				} catch (IOException e1) {
					e1.printStackTrace();
				}
			//	setResponse(response, "{success: false, message: \"Falsches Dateiformat hochgeladen!\"}");
				e.printStackTrace();
			}
		} else {
			RequestType requestType = getRequestType(request);

			if (requestType.equals(RequestType.getAllStudies)){
				logger.info("getAllStudies requested");
				List<Study> studies = StudyManagement.getAllStudies();

				JSONObject jo = new JSONObject();
				JSONArray ja = new JSONArray();
				for(Study study : studies){
					JSONObject tmp = new JSONObject();
					tmp.element("id", study.getId());
					tmp.element("name", study.getName());
					tmp.element("active", study.getActive());
					if(study.getIdentifier().length() >0)
						tmp.element("identifier", " ("+study.getIdentifier()+")");
					else
						tmp.element("identifier", "");
					tmp.element("description", study.getDescription());

					JSONArray snps = new JSONArray();
					for (Snp snp : study.getSnps()) {
						JSONObject s = new JSONObject();
						s.element("name", snp.getName());
						s.element("id", snp.getId());
						s.element("directory", snp.getDirectory());
						s.element("sex", snp.getSex());
						if  (snp.getSexsnp().isEmpty())
							s.element("sexsnp", 0);
						else
							s.element("sexsnp", 1);
						
						//--- LF-2012-06-19
						
						s.element("study", snp.getStudy().getId());
						
						//--- LF-2012-06-19
						
						snps.add(s);
					}
					tmp.element("snps", snps);
					ja.add(tmp);
				}
				jo.element("data", ja);
				setResponse(response, jo.toString());
			}
			else if (requestType.equals(RequestType.getSamplesFromStudy)){
				String studyId = request.getParameter("study.id");
				String min = request.getParameter("start");
				String max = request.getParameter("limit");
				JSONObject jo = StudyManagement.getSamplesFromStudyFetch(studyId, min, max);
				setResponse(response, jo.toString());
			}
			else if (requestType.equals(RequestType.getStudy)){
				String studyId = request.getParameter("studyId");
				JSONObject jo = StudyManagement.getStudy(studyId);
				setResponse(response, jo.toString());
			}
			else if (requestType.equals(RequestType.activateStudy)){
				String studyId = request.getParameter("studyId");
				Study study = StudyManagement.getStudyById(studyId);
				StudyManagement.activateStudy(study);

				String result = "{success: true, message: \"lbActivateStudySuccess\"}";
				setResponse(response, result);
			}
			else if (requestType.equals(RequestType.deactivateStudy)){
				String studyId = request.getParameter("studyId");
				Study study = StudyManagement.getStudyById(studyId);
				StudyManagement.deactivateStudy(study);

				String result = "{success: true, message: \"lbDeactivateStudySuccess\"}";
				setResponse(response, result);
			}
			else if (requestType.equals(RequestType.getSampleIds)){
				String studyId = request.getParameter("study.id");
				String result = StudyManagement.getSampleIds(studyId);
				setResponse(response, result);
			}
			else if (requestType.equals(RequestType.updateSample)){
				String id = request.getParameter("id");
				Sample sample = StudyManagement.getSampleById(id);
				String concentration = request.getParameter("concentration");
				String qualityValue = request.getParameter("qualityValue");
				String tmpIsError = request.getParameter("isError");
				boolean isError = (tmpIsError != null) && tmpIsError.equalsIgnoreCase("true");
				sample.setIsError(isError);
				sample.setConcentration(concentration);
				sample.setQualityValue(qualityValue);

				String result = StudyManagement.updateSample(sample);
				setResponse(response, result);
			}
			else if(requestType.equals(RequestType.insertStudyMin)){
				String studyName = request.getParameter("study.name");
				String studyIdentifier = request.getParameter("study.identifier");
				String studyDescription = request.getParameter("study.description");

				Study study = new Study();
				study.setName(studyName);
				study.setIdentifier(studyIdentifier);
				study.setDescription(studyDescription);

				StudyManagement.createStudy(study);
				String result = "{success: true}";
				setResponse(response, result);
			}
			else if(requestType.equals(RequestType.editStudy)){
				String studyId = request.getParameter("id");
				String studyName = request.getParameter("study.name");
				String studyIdentifier = request.getParameter("study.identifier");
				String studyDescription = request.getParameter("study.description");
				String numMultiple = request.getParameter("study.numMultiple");

				Study study = StudyManagement.getStudyById(studyId);
				study.setId(Integer.parseInt(studyId));
				study.setName(studyName);
				study.setIdentifier(studyIdentifier);
				study.setDescription(studyDescription);
				study.setNumMultiple(Integer.parseInt(numMultiple));

				StudyManagement.updateStudy(study);
				String result = "{success: true}";
				setResponse(response, result);
			}
			else
				throw new IllegalArgumentException("Request Type \"" + requestType + "\" is not supported");
		}
	}

	private void handleFileUpload(HttpServletRequest request) throws FileUploadException, IOException{
		// Create a new file upload handler
		ServletFileUpload upload = new ServletFileUpload();

		// Parse the request
		FileItemIterator iter = upload.getItemIterator(request);
		String studyName = null;
		String studyDescription = null;
		String studyIdentifier = null;
		int numMultiple = 0;
		List<String> sampleIds = null;

		while (iter.hasNext()) {
		    FileItemStream item = iter.next();
		    String fieldName = item.getFieldName();
		    InputStream stream = item.openStream();
		   
		    if(fieldName.equals("study.name"))
		    	studyName = Streams.asString(stream);
		    else if(fieldName.equals("study.description"))
		    	studyDescription = Streams.asString(stream);
		    else if(fieldName.equals("study.identifier"))
		    	studyIdentifier = Streams.asString(stream);
		    else if(fieldName.equals("study.numMultiple")){
		    	String field=Streams.asString(stream);
		   	if (field.isEmpty())
		    	numMultiple = 0;
		    else{
		    	numMultiple = Integer.parseInt(field);}
		    
		    }
		    if (!item.isFormField()) {
		    
		    	sampleIds = ProbeImporter.parseSampleIds(stream, item.getContentType());
		    
		    	if(stream != null)
		    		stream.close();
		    }
		}

		Study study = new Study();
		study.setName(studyName);
		study.setIdentifier(studyIdentifier);
		study.setDescription(studyDescription);
		study.setNumMultiple(numMultiple);
		study.setActive(true);
		Study insertedStudy = StudyManagement.createStudy(study);


		if(sampleIds != null)
			StudyManagement.importSampleIds(sampleIds, Integer.valueOf(insertedStudy.getId()));
	}

	private RequestType getRequestType(HttpServletRequest request) {
		String requestType = request.getParameter("requestType");
		return RequestType.byValue(requestType);
	}
}
