Ext.namespace('workflow');

workflow.SelectionBox = function(config) {
    Ext.apply(this, config, {});

    var me = this;

    this.store = new Ext.data.Store({
	reader : new Ext.data.JsonReader({
	    id : 'name',
	    root : 'data',
	    totalProperty : 'totalCount',
	    fields : [ {
		name : 'name',
		type : 'string'
	    }, {
		name : 'value',
		type : 'string'
	    }, {
		name : 'nums',
		type : 'int'
	    }, {
		name : 'snpName',
		type : 'string'
	    }, {
		name : 'isOk',
		type : 'int'
	    } ]
	}),
	autoLoad : false,
	proxy : new Ext.data.HttpProxy({
	    url : './SNPDataServlet'
	}),
	baseParams : {
	    studyId : me.studyId,
	    requestType : 'getDetectorNames'
	},
	listeners : {
	    load : function(store, records) {
		console.log('table store loaded successfull.');
		var value = records[0].json.snpName;
		var value2 = records[1].json.snpName;
		if (value.length >= value2.length) {
		    Ext.getCmp('snpName').setValue(value);
		    Ext.getCmp('hapmapName').setValue(value);
		} else {
		    Ext.getCmp('snpName').setValue(value2);
		    Ext.getCmp('hapmapName').setValue(value2);
		}
		var valueCheck = records[0].json.isOk;

		Ext.getCmp('snpComment').setValue('');
		Ext.getCmp('checkbox').setValue(valueCheck);
		workflow.LoadMask.hide();
		me.wizRef.nextButton.setDisabled(true);

		Ext.getCmp('checkbox').setDisabled(false);

		Ext.getCmp('snpName').setDisabled(false);

		Ext.getCmp('hapmapName').setDisabled(false);

		Ext.getCmp('snpInsertedBy').setDisabled(false);

		Ext.getCmp('snpComment').setDisabled(false);
	    }
	}
    });

    var bobar = new Ext.PagingToolbar({
	name : 'bobar',
	id : 'bobar',
	pageSize : 1,
	store : this.store,
	emptyMsg : "No snps to display",
	nextText : 'next'
    /*
     * doRefresh : function() { this.doLoad(this.cursor); }
     */
    });

    var checkMandatorySNP = function(name) {
	if (name == null || name.length < 1) {
	    return false;
	}
	return true;
    };

    var checkMandatoryName = function(insertedBy) {
	if (insertedBy == null || insertedBy.length < 1) {
	    return false;
	}
	return true;
    };

    var checkDetectorNames = function() {
	var count = 0;
	var und = 'Undetermined';
	var empty = myLocale.lbEmptyWells;

	me.store.data.each(function(item, index, total) {
	    var key = item.data['name'];
	    var value = item.data['value'];

	    if (key != und && key != empty && value.length > 0)
		++count;
	});

	return count == 3;
    };

    var refresh = function() {
	bobar.doRefresh();
    };

    this.bottomGrid = new Ext.grid.EditorGridPanel({
	name : 'bottomgrid',
	autoScroll : true,
	frame : true,
	height : 145,
	margins : '5 5 5 5',
	width : '100%',
	striperows : true,
	autoHeight : true,
	autoScroll : true,
	// loadMask: true,
	stripeRows : true,
	store : this.store,
	columns : [ {
	    header : myLocale.lbDetectors,
	    width : 200,
	    sortable : false,
	    dataIndex : 'name',
	    editor : new Ext.form.TextField()
	}, {
	    header : myLocale.lbWillReplace,
	    width : 150,
	    sortable : false,
	    dataIndex : 'value',
	    editor : new Ext.form.TextField()
	}, {
	    header : myLocale.lbCount,
	    width : 150,
	    sortable : false,
	    dataIndex : 'nums'
	}
	// {header: 'Snp', width: 150, sortable: true, dataIndex: 'snpName'}
	],

	// paging bar on the bottom
	bbar : bobar,

	listeners : {
	    rowclick : function(g, index, ev) {
		var rec = g.store.getAt(index);
	    }
	}
    });

    this.namingSet = new Ext.form.FieldSet(
	    {
		border : false,
		frame : false,
		hideLabel : true,
		layout : 'column',
		items : [
			{
			    columnWidth : 0.32,
			    layout : 'form',
			    border : false,
			    items : [ new Ext.form.Label({
				fieldLabel : myLocale.lbNameSnp
			    }), new Ext.form.Label({
				fieldLabel : myLocale.lbHapMapLookup 
			    }), new Ext.form.Label({
				fieldLabel : myLocale.lbCreator
			    }), new Ext.form.Label({
				fieldLabel : myLocale.lbComment
			    }) ]
			},
			{
			    columnWidth : 0.68,
			    layout : 'form',
			    border : false,
			    items : [
				    {
					xtype : 'textfield',
					id : 'snpName',
					name : 'snpName',
					emptyText : myLocale.lbMandatory1,
					width : 250,
					tabIndex : 1,
					hideLabel : true
				    },
				    {
					xtype : 'textfield',
					id : 'hapmapName',
					name : 'hapmapName',
					emptyText : myLocale.lbHapMapName,
					width : 250,
					tabIndex : 2,
					hideLabel : true
				    },
				    {
					xtype : 'textfield',
					id : 'snpInsertedBy',
					name : 'snpInsertedBy',
					emptyText : myLocale.lbCreator,
					width : 250,
					tabIndex : 3,
					hideLabel : true
				    },
				    {
					xtype : 'textfield',
					id : 'snpComment',
					name : 'snpComment',
					tabIndex : 4,
					emptyText : myLocale.lbEmptyComment,
					width : 250,
					hideLabel : true
				    },
				    {
					xtype : 'checkbox',
					id : 'checkbox',
					name : 'checkbox',
					labelSeparator : '',
					hideLabel : true,
					boxLabel : myLocale.lbConfirmInput,
					fieldLabel : myLocale.lbConfirmInput,
					listeners : {
					    check : function(box, checked) {
						if (checked == true) {

						    // disables all components
						    Ext.getCmp('checkbox')
							    .setDisabled(true);

						    Ext.getCmp('snpName')
							    .setDisabled(true);

						    Ext.getCmp('hapmapName')
							    .setDisabled(true);

						    Ext.getCmp('snpInsertedBy')
							    .setDisabled(true);

						    Ext.getCmp('snpComment')
							    .setDisabled(true);

						    // gets input data

						    var snpName = Ext.getCmp(
							    'snpName')
							    .getValue();
						    var hapmapName = Ext
							    .getCmp(
								    'hapmapName')
							    .getValue();
						    var snpInsertedBy = Ext
							    .getCmp(
								    'snpInsertedBy')
							    .getValue();
						    var snpComment = Ext
							    .getCmp(
								    'snpComment')
							    .getValue();
						    var holder = [];
						    me.store.data
							    .each(function(
								    item,
								    index,
								    total) {
								var key = item.data['name'];
								var value = item.data['value'];
								holder
									.push({
									    key : key,
									    value : value
									});
							    });

						    // error checking

						    if (!checkMandatorySNP(snpName)) {
							Ext.Msg
								.alert(
									myLocale.lbError,
									myLocale.lbMandatory1);
							Ext
								.getCmp(
									'checkbox')
								.setValue(false);
							Ext.getCmp('checkbox')
								.setDisabled(
									false);

							// enables all
							// components

							Ext.getCmp('snpName')
								.setDisabled(
									false);

							Ext
								.getCmp(
									'hapmapName')
								.setDisabled(
									false);

							Ext
								.getCmp(
									'snpInsertedBy')
								.setDisabled(
									false);

							Ext
								.getCmp(
									'snpComment')
								.setDisabled(
									false);

							return;
						    } else if (!checkMandatoryName(snpInsertedBy)) {
							Ext.Msg
								.alert(
									myLocale.lbError,
									myLocale.lbMandatory2);
							Ext
								.getCmp(
									'checkbox')
								.setValue(false);
							Ext.getCmp('checkbox')
								.setDisabled(
									false);
							// enables all
							// components

							Ext.getCmp('snpName')
								.setDisabled(
									false);

							Ext
								.getCmp(
									'hapmapName')
								.setDisabled(
									false);

							Ext
								.getCmp(
									'snpInsertedBy')
								.setDisabled(
									false);

							Ext
								.getCmp(
									'snpComment')
								.setDisabled(
									false);

							return;
						    } else if (!checkDetectorNames(
							    snpName,
							    snpInsertedBy)) {
							Ext.Msg
								.alert(
									myLocale.lbError,
									myLocale.lbMandatory3);
							Ext
								.getCmp(
									'checkbox')
								.setValue(false);
							Ext.getCmp('checkbox')
								.setDisabled(
									false);

							// enables all
							// components

							Ext.getCmp('snpName')
								.setDisabled(
									false);

							Ext
								.getCmp(
									'hapmapName')
								.setDisabled(
									false);

							Ext
								.getCmp(
									'snpInsertedBy')
								.setDisabled(
									false);

							Ext
								.getCmp(
									'snpComment')
								.setDisabled(
									false);

							return;
						    } else {
							var start = bobar
								.getPageData().activePage;
							var end = bobar
								.getPageData().total;

							if (end > 1) {
							    Ext.Ajax
								    .request({
									url : './SNPDataServlet',
									params : {
									    requestType : 'removeSNPfromHashList',
									    start : start,
									    studyId : me.studyId,
									    snpName : snpName,
									    hapmapName : hapmapName,
									    snpInsertedBy : snpInsertedBy,
									    snpComment : snpComment,
									    replacements : Ext.util.JSON
										    .encode(holder)
									},
									success : function(
										response) {
									    me.store
										    .reload();
									}
								    });

							} else if (end == 1) {
							    Ext.Ajax
								    .request({
									url : './SNPDataServlet',
									params : {
									    requestType : 'removeSNPfromHashList',
									    studyId : me.studyId,
									    snpName : snpName,
									    hapmapName : hapmapName,
									    snpInsertedBy : snpInsertedBy,
									    snpComment : snpComment,
									    replacements : Ext.util.JSON
										    .encode(holder)
									},
									success : function(
										response) {
									    console
										    .log('remove snp successfull.');
									    me.store
										    .reload();
									}
								    });

							    me.wizRef.nextButton
								    .setDisabled(false);
							}

						    }

						    /*
						     * setTimeout(function() {
						     * Ext.getCmp('checkbox')
						     * .setDisabled( false); },
						     * 500);
						     */

						}
					    }
					}
				    } ]
			} ]
	    });

    workflow.SelectionBox.superclass.constructor.call(this, Ext.apply({
	name : 'super',
	layout : 'form',
	frame : true,
	border : true,
	labelWidth : 150,
	title : myLocale.lbDetectorSettings,

	items : [ this.namingSet, this.bottomGrid ]
    }));
};

Ext.extend(workflow.SelectionBox, Ext.ux.Wiz.Card, {
    wizRef : null,

    studyId : null,

    res : null,

    initWizCard : function() {
	workflow.LoadMask.show();
	this.store.load({
	    params : {
		start : 0,
		limit : 1
	    }
	});
    },

    commitCard : function(respHandler) {

	var me = this;
	workflow.LoadMask.show();

	Ext.Ajax.timeout = 180000; // 3 minutes

	Ext.Ajax.request({
	    url : './SNPDataServlet',
	    success : function(response) {
		workflow.LoadMask.hide();
		var obj = Ext.util.JSON.decode(response.responseText);
		me.wizRef.snpId = obj.snpId;
		respHandler(me.wizRef);
	    },
	    failure : function() {
		workflow.LoadMask.hide();
	    },
	    params : {
		requestType : 'calculate',
		studyId : this.studyId
	    }
	});

    }
});