Ext.namespace('workflow');

workflow.SexSnpSelection = function(config) {
	Ext.apply(this, config,{});

	var me = this;

	
	this.store = new Ext.data.Store({
        reader: new Ext.data.JsonReader({
			id: 'name',
			root: 'data',
			totalProperty:'totalCount',
			fields:[{
			    name: 'genotype',
			    type: 'string'
			}, {
			    name: 'phenotype',
			    type: 'string'
			}
			]
        }),
        id: 'store',
        proxy: new Ext.data.HttpProxy({url:'./SNPDataServlet'}),
        baseParams: {snpId: me.snpId, requestType: 'getSex'},
        listeners: {
        	load: function(store, records){
        		var valueG = records[0].json.genotype;
        		var valueG2 = records[1].json.genotype;
        	
        		if (records.length == 2)
        		{
        		var valueP = records[0].json.phenotype;
        		var valueP2 = records[1].json.phenotype;
        		workflow.LoadMask.hide();
        		Ext.getCmp('MaleGenotype').setValue(valueG);
        		Ext.getCmp('MalePhenotype').setValue(valueP);
        		Ext.getCmp('FemaleGenotype').setValue(valueG2);
        		Ext.getCmp('FemalePhenotype').setValue(valueP2);
        		}
        		else
        		{
        			me.wizRef.nextButton.setDisabled(true);
        			   			Ext.MessageBox.show({
            		           title: 'Not a proper Sex-SNP',
            		           msg: 'The selected SNP is not a proper Sex-SNP!<br/> Found: '+records[0].json.genotype+' ' + records[1].json.genotype + ' ' +records[2].json.genotype,
            		           buttons: Ext.MessageBox.OK
             		       });
        			   			me.wizRef.close();	
        		}
         	}
        }
    });



		this.namingSet = new Ext.form.FieldSet({
		border : false,
		frame : false,
		hideLabel : true,
		layout : 'column',
		items : [ {
			columnWidth : 0.33,
			layout : 'form',
			border : false,
			items : [ new Ext.form.Label({fieldLabel : 'Sex'	}),
			          new Ext.form.Label({	fieldLabel : 'Male'	}), 
			          new Ext.form.Label({fieldLabel : 'Female'	}) 
			]
		}, {
			columnWidth : 0.33,
			layout : 'form',
			border : false,
			items : [  new Ext.form.Label({fieldLabel : 'Genotype'	}) , 
			          new Ext.form.ComboBox({
				store : this.store,
				displayField : 'genotype',
				typeAhead : true,
				width : 100,
				hideLabel : true,
				mode : 'local',
				id : 'MaleGenotype',
				triggerAction : 'all',
				selectOnFocus : true,
				listeners : {
					scope : this,
					change : function(combo, newVal, oldVal) {

						console.log(newVal + ' ' + oldVal );
						Ext.getCmp('FemaleGenotype').setValue(oldVal);
					}
				}

			}), new Ext.form.ComboBox({
				store : this.store,
				displayField : 'genotype',
				typeAhead : true,
				width : 100,
				hideLabel : true,
				mode : 'local',
				id : 'FemaleGenotype',
				triggerAction : 'all',
				selectOnFocus : true,
				listeners : {
					scope : this,
					change : function(combo, value, index) {
						console.log(value + ' ' + index);
						Ext.getCmp('MaleGenotype').setValue(index);
					}
				}
			})

			]
		}, {
			columnWidth : 0.33,
			layout : 'form',
			border : false,

			items : [ new Ext.form.Label({fieldLabel : 'Phenotype'	}),
			          new Ext.form.ComboBox({
				store : this.store,
				displayField : 'phenotype',
				typeAhead : true,
				width : 100,
				hideLabel : true,
				mode : 'local',
				id : 'MalePhenotype',
				triggerAction : 'all',
				selectOnFocus : true,
				listeners : {
					scope : this,
					change : function(combo, value, index) {
						console.log(value + ' ' + index);
						Ext.getCmp('FemalePhenotype').setValue(index);
					}
				}
			}), new Ext.form.ComboBox({
				store : this.store,
				displayField : 'phenotype',
				typeAhead : true,
				width : 100,
				hideLabel : true,
				mode : 'local',
				id : 'FemalePhenotype',
				triggerAction : 'all',
				selectOnFocus : true,
				listeners : {
					scope : this,
					change : function(combo, value, index) {
						console.log(value + ' ' + index + ' ' + combo.getValue());
						Ext.getCmp('MalePhenotype').setValue(index);
					}
				}
			}) ]
		}

		]
	});
		
	
	
workflow.SexSnpSelection.superclass.constructor.call(this, Ext.apply({
		name: 'super',
		layout: 'form',
		frame: true,
		border: true,
		labelWidth: 150,
		title: 'Define genotype & phenotype sex',
		items: [ this.namingSet]
	}));
};

Ext.extend(workflow.SexSnpSelection, Ext.ux.Wiz.Card, {
	wizRef: null,

	snpId: null,
	
	res: null,

	initWizCard: function(){
		//workflow.LoadMask.show();
		
		
	},

	commitCard: function(respHandler){
	
		var me = this;
		workflow.LoadMask.show();
		
		Ext.Ajax.timeout = 180000; //3 minutes

		Ext.Ajax.request({
    		url: './SNPDataServlet',
    		success: function(response){
				workflow.LoadMask.hide();
				var obj = Ext.util.JSON.decode(response.responseText);
				me.wizRef.snpId = obj.snpId;
				if (obj.success == false)
					Ext.MessageBox.alert('Failure Sex SNP '+ obj.snpId, 'Error - more than 50% wrong');
				else{
				respHandler(me.wizRef);
	       		workflow.selectSnp(me.wizRef.snpId);
        		workflow.study.getStudies(me.wizRef.snpId);
				}
			},
    		failure: function(){
				workflow.LoadMask.hide();
				var obj = Ext.util.JSON.decode(response.responseText);
				me.wizRef.snpId = obj.snpId;
				Ext.MessageBox.alert('failure sexsnp '+ obj.snpId );
			},
    		params: {
    			requestType: 'calculateSexSnp',
    			snpId: this.snpId,
    			maleG:  Ext.getCmp('MaleGenotype').getValue(),
    			femaleG:Ext.getCmp('FemaleGenotype').getValue(),
    			maleP:  Ext.getCmp('MalePhenotype').getValue(),
    			femaleP:Ext.getCmp('FemalePhenotype').getValue()
     		}
    	});

	}
});