Ext.namespace('workflow');

workflow.Viewport = function(config) {
    Ext.apply(this, config, {});

    this.editStudyItem = new Ext.menu.Item({
	icon : './images/edit.png',
	text : myLocale.lbEdit + '...',
	disabled : true,
	tooltip : myLocale.lbEditSelectedStudy,
	scope : this,
	handler : function() {
	    this.editStudy();
	}
    });
    this.createSnpItem = new Ext.menu.Item({
	text : myLocale.lbCreateSnp,
	disabled : true,
	icon : './images/calculation.png',
	scope : this,
	handler : function() {
	    this.createSnp();
	}
    });
    this.activateStudyItem = new Ext.menu.Item({
	cls : 'bactivate',
	text : myLocale.lbActivate,
	disabled : true,
	tooltip : myLocale.lbActivateStudy,
	scope : this,
	handler : function() {
	    this.activateStudy();
	}
    });
    this.deactivateStudyItem = new Ext.menu.Item({
	cls : 'bdeactivate',
	text : myLocale.lbDeactivate,
	disabled : true,
	tooltip : myLocale.lbDeactivateStudy,
	scope : this,
	handler : function() {
	    this.deactivateStudy();
	}
    });

    this.mainMenu = new Ext.menu.Menu({
	items : [ new Ext.menu.Item({
	    cls : 'bimport',
	    text : myLocale.lbAdd + '...',
	    disabled : !workflow.user.admin,
	    tooltip : myLocale.lbNewStudyDescription,
	    handler : workflow.study.create
	}), this.editStudyItem, '-', this.createSnpItem, '-',
		this.activateStudyItem, this.deactivateStudyItem ]
    });

    this.exportSnpItem = new Ext.menu.Item({
	text : myLocale.lbExport + '...',
	icon : './images/export.png',
	disabled : true,
	scope : this,
	handler : function() {
	    this.exportSnp();
	}
    });

    this.setSexSnp = new Ext.menu.Item({
	text : myLocale.lbSexSnp + '...',
	icon : './images/users.png',
	disabled : true,
	scope : this,
	handler : function() {
	    this.exportSnp();
	}
    });

    this.deleteSnpItem = new Ext.menu.Item({
	text : myLocale.lbDelete + '...',
	icon : './images/delete.png',
	disabled : true,
	scope : this,
	handler : function() {
	    this.deleteSnp();
	}
    });

    this.editMenu = new Ext.menu.Menu({
	items : [ this.exportSnpItem, this.setSexSnp, '-', this.deleteSnpItem ]
    });

    // this.exMenu = new Ext.menu.Menu({
    // items: [
    // new Ext.menu.Item({
    // cls: 'busers',
    // text: myLocale.lbUserManagement + '...',
    // tooltip: myLocale.lbCrudUser,
    // disabled: !workflow.user.admin,
    // handler: function() {
    // var us = new workflow.UserManagement();
    // us.show();
    // us.center();
    // }
    // })]
    // });

    this.helpMenu = new Ext.menu.Menu(
	    {
		items : [
			new Ext.menu.Item(
				{
				    cls : 'mhelp',
				    text : 'User guide',
				    tooltip : 'User guide',
				    handler : function() {
					window
						.open("http://www3.i-med.ac.at/genepi-snpflow/userguide.html");

				    }
				}),
			new Ext.menu.Item(
				{
				    cls : 'minfo',
				    text : 'SNPflow Project',
				    tooltip : 'About SNPflow',
				    handler : function() {
					window
						.open("http://www3.i-med.ac.at/genepi-snpflow");
				    }
				}) ]
	    });

    this.workflowTb = new Ext.Toolbar('toolbar');
    this.workflowTb.add({
	cls : 'x-btn-text-icon bstudy',
	text : myLocale.lbStudy,
	menu : this.mainMenu
    });
    this.workflowTb.add({
	cls : 'x-btn-text-icon bsnps',
	text : myLocale.lbSnp,
	menu : this.editMenu
    });
    this.workflowTb.add({
	cls : 'x-btn-text-icon busers',
	icon : './images/users.png',
	text : myLocale.lbUserManagement,
	tooltip : myLocale.lbCrudUser,
	disabled : !workflow.user.admin,
	handler : function() {
	    var us = new workflow.UserManagement();
	    us.show();
	    us.center();
	}
    });
    this.workflowTb.add('-');
    this.workflowTb.add({
	cls : 'x-btn-text-icon bhelp',
	text : myLocale.lbHelp,
	menu : this.helpMenu
    });
    this.workflowTb.add('->');
    this.workflowTb.add({
	cls : 'x-btn-text-icon bexit',
	text : myLocale.lbLogout,
	handler : this.logout
    });

    this.studyTree = new Ext.tree.TreePanel({
	region : 'west',
	id : 'west-panel',
	collapsible : true,
	title : myLocale.lbStudyOverview,
	width : 190,
	minWidth : 190,
	autoScroll : true,
	split : true,
	selModel : new Ext.tree.MultiSelectionModel(),
	collapseMode : 'mini',
	hideCollapseTool : true,
	animate : true,
	containerScroll : true,
	rootVisible : false,
	border : false,
	root : {
	    text : myLocale.lbStudies,
	    id : 'rootId',
	    expanded : true,
	    children : []
	},
	tbar : [ {
	    xtype : 'trigger',
	    emptyText : myLocale.lbSearch + '...',
	    triggerClass : 'x-form-clear-trigger',
	    width : 185,
	    me : this,
	    scope : this,
	    onTriggerClick : function() {
		this.reset();
		this.me.studyTree.filter.clear();
		this.me.studyTree.collapseAll();

	    },
	    enableKeyEvents : true,
	    listeners : {
		keyup : {
		    buffer : 300,
		    scope : this,
		    fn : function(field, e) {
			var val = field.getRawValue();
			this.studyTree.filter.clear();
			this.studyTree.filter.filterBy(function(n) {
			    var doFilter = true;

			    if (val.length > 0) {
				var re = new RegExp('.*' + val + '.*', 'i');
				doFilter = re.test(n.attributes.text);
				if (!doFilter) {
				    n.expand();
				}
			    }

			    return doFilter;
			});
		    }
		}
	    }
	} ]
    });
    this.studyTree.filter = new Ext.ux.tree.TreeFilterX(this.studyTree);

    this.studyTree.addListener('click', function(node, event) {

	if (node.isLeaf() && node.attributes.name !== 'show'
		&& node.attributes.name !== 'inactive') {
	    this.snpSelected();
	} else {

	    if (node.parentNode.isRoot) {

		this.studySelected();

	    } else {

		this.snpFolderSelected();

	    }
	}
    }, this);

    workflow.Viewport.superclass.constructor
	    .call(
		    this,
		    Ext
			    .apply({
				layout : 'border',
				loadMask : {
				    msg : myLocale.lbBuildingPage
				},
				items : [
					new Ext.Panel({
					    region : 'north',
					    tbar : this.workflowTb,
					    margins : '0 0 0 0',
					    height : 25
					}),
					this.studyTree,
					{
					    region : 'center',
					    layout : 'fit',
					    id : 'center-panel',
					    border : false,
					    frame : false,
					    html : '<div id="wrapper"><div id="container"><center>'
						    + '<iframe src="text.htm" width="100%" height="750" name="" style="border: 0px"></iframe>'
						    + '<br /><br />'
						    + '</center></div></div>'
					} ]
			    }));
};

Ext.extend(workflow.Viewport, Ext.Viewport, {
    logout : function() {
	Ext.Msg.confirm(myLocale.lbLogout, myLocale.lbReallyLogout, function(
		btn, text) {
	    if (btn == 'yes') {
		Ext.Ajax.request({
		    url : './AuthenticationServlet',
		    success : window.location = 'index.jsp',
		    failure : function() {
		    },
		    params : {
			logout : true
		    }
		});
	    }
	});
    },

    exportSnp : function() {
	var siblings = Ext.getCmp('west-panel').getSelectionModel()
		.getSelectedNodes();
	var snpId = "";
	for ( var i = 0; i < siblings.length; i++) {

	    try {
		if (siblings[i].id.indexOf("snp") != -1)
		    snpId = snpId + " " + siblings[i].id.split("_")[1];

		snpId = snpId + " " + siblings[i].id.split("_")[1];
	    } catch (err) {
		Ext.Msg.alert('Attention', 'Please select SNPs only!');
	    }
	}
	if (snpId.trim().length == 0) {
	    siblings[0].select();
	    snpName = siblings[0].text;
	    snpId = siblings[0].id.split("_"[1]);
	}

	var win = new workflow.ExportSnp({
	    snpId : snpId,
	    multi : siblings[0].id
	});
	win.show();
	win.center();
    },

    createSnp : function() {
	var node = this.studyTree.getSelectionModel().getSelectedNodes();
	var studyID = node[0].id;

	var win = new workflow.ImportWizard({
	    studyId : studyID
	});
	win.show();
	win.center();
    },

    editStudy : function() {
	var node = this.studyTree.getSelectionModel().getSelectedNodes();
	var studyID = node[0].id;
	workflow.editStudy(studyID);
    },

    deactivateStudy : function() {

	var node = this.studyTree.getSelectionModel().getSelectedNodes();
	var studyID = node[0].id;
	var me = this;
	Ext.Msg.confirm(myLocale.lbConfirm, myLocale.lbReallyDeactivateStudy,
		function(btn, text) {
		    if (btn == 'yes') {
			workflow.LoadMask.show();
			Ext.Ajax.request({
			    url : './WorkflowServlet',
			    success : function(response) {
				me.clearSelected();
				workflow.study.getStudies();
				workflow.successHandlingAjax(response);
			    },
			    failure : function(response) {
				workflow.LoadMask.hide();
			    },
			    params : {
				requestType : 'deactivateStudy',
				studyId : studyID
			    }
			});
		    }
		});
    },

    activateStudy : function() {
	var node = Ext.getCmp('west-panel').getSelectionModel()
		.getSelectedNodes();
	var studyID = node[0].id;
	var me = this;
	Ext.Msg.confirm(myLocale.lbConfirm, myLocale.lbReallyActivateStudy,
		function(btn, text) {
		    if (btn == 'yes') {
			workflow.LoadMask.show();
			Ext.Ajax.request({
			    url : './WorkflowServlet',
			    success : function(response) {
				me.clearSelected();
				workflow.study.getStudies();
				workflow.successHandlingAjax(response);
			    },
			    failure : function(response) {
				workflow.LoadMask.hide();
			    },
			    params : {
				requestType : 'activateStudy',
				studyId : studyID
			    }
			});
		    }
		});
    },

    deleteSnp : function() {
	var siblings = this.studyTree.getSelectionModel().getSelectedNodes();

	var snpId = "";

	var snpName = "";
	try {
	    snpId = siblings[0].id.split("_")[1];
	    for ( var i = 0; i < siblings.length; i++) {

		snpName = snpName + " " + siblings[i].text;
		Ext.Msg.alert('Attention', siblings[i].id);
		snpId = snpId + " " + siblings[i].id.split("_")[1];
	    }
	} catch (err) {
	    Ext.Msg.alert('Attention', 'Please select SNPs only!');
	}

	if (snpId.trim().length == 0) {
	    siblings[0].select();
	    snpName = siblings[0].text;
	    snpId = siblings[0].id.split("_"[1]);
	}

	Ext.Msg.confirm(myLocale.lbConfirmDelete, myLocale.lbReallyDeleteSnp
		+ '<br />' + snpName.trim(), function(btn, text) {
	    if (btn == 'yes') {
		workflow.LoadMask.show();
		Ext.Ajax.request({
		    url : './SNPDataServlet',
		    success : function(response) {
			for ( var i = 0; i < siblings.length; i++) {
			    siblings[i].parentNode.removeChild(siblings[i]);
			}
			workflow.successHandlingAjax(response);
		    },
		    failure : function(response) {
			workflow.LoadMask.hide();
		    },
		    params : {
			requestType : 'deleteSNP',
			snpId : snpId,
			multi : 'null'
		    }
		});
	    }
	});

    },

    clearSelected : function() {
	this.exportSnpItem.setDisabled(true);
	this.deleteSnpItem.setDisabled(true);

	this.editStudyItem.setDisabled(true);
	this.createSnpItem.setDisabled(true);
	this.activateStudyItem.setDisabled(true);
	this.deactivateStudyItem.setDisabled(true);
    },

    snpSelected : function() {
	this.exportSnpItem.setDisabled(false);
	this.deleteSnpItem.setDisabled(!(workflow.user.admin || workflow.user.lab));
	this.editStudyItem.setDisabled(true);
	this.createSnpItem.setDisabled(true);
	this.setSexSnp.setDisabled(!(workflow.user.admin || workflow.user.lab));
	this.activateStudyItem.setDisabled(true);
	this.deactivateStudyItem.setDisabled(true);
    },

    studySelected : function() {
	this.exportSnpItem.setDisabled(true);
	this.deleteSnpItem.setDisabled(true);

	this.editStudyItem.setDisabled(!(workflow.user.admin || workflow.user.lab));
	this.createSnpItem
		.setDisabled(!(workflow.user.admin || workflow.user.lab));
	this.setSexSnp.setDisabled(true);
	this.activateStudyItem.setDisabled(!workflow.user.admin);
	this.deactivateStudyItem.setDisabled(!workflow.user.admin);
    },
    snpFolderSelected : function() {
	this.exportSnpItem.setDisabled(true);
	this.deleteSnpItem.setDisabled(true);

	this.editStudyItem.setDisabled(true);
	this.createSnpItem.setDisabled(true);
	this.setSexSnp.setDisabled(true);
	this.activateStudyItem.setDisabled(true);
	this.deactivateStudyItem.setDisabled(true);
    }
});