Ext.namespace('Ext.Bugfixes');

Ext.Bugfixes.init = function () {

	/*
	 * Bugfix for fixing the tab order in modal windows. Remove when its not needed anymore.
	 * Link: http://extjs.com/forum/showthread.php?t=29231
	 */
	Ext.override(Ext.Window, {
	   handleTabs : true,
	
	   initEvents : Ext.Window.prototype.initEvents.createInterceptor(function () {
	      if (this.handleTabs === true) {
	         this.keys = [].concat(this.keys || [], { key : Ext.EventObject.TAB, fn : this.onTAB, scope : this });
	      }
	   }),
	
	   onTAB : function (keyCode, e) {
	      e.stopEvent();
	      
	      // TAB moves forward through the order. SHIFT-TAB moves backward through the order.
	      var backward = e.shiftKey;
	
	      // Get the set of controls that have a defined tabIndex >= 0. Sort them ascending or descending depending
	      // on whether we are moving the focus forward or backward.
	      var elements = Ext.query('[tabIndex][tabIndex!=-1]', this.el.dom).sort(function (a, b) {
	         return backward ? b.tabIndex - a.tabIndex : a.tabIndex - b.tabIndex;
	      });
	      
	      // We cannot determine the next element in the order if there are no elements with a defined tabIndex.
	      if (elements.length === 0) {
	         return;
	      }
	      
	      // Get the tabIndex from the control that had focus when TAB/SHIFT-TAB was pressed
	      var currentIndex = e.getTarget().tabIndex || 0;
	
	      // Check each element to see if it is the next one in the ordering      
	      for (var i=0; i<elements.length; i++) {
	         var el    = elements[i];
	         var index = el.tabIndex;
	         
	         if ((backward && index < currentIndex) || (!backward && index > currentIndex)) {
	            el.focus();
	            return;
	         }
	      }
	      
	      // Since no element was the next in the ordering, loop around to other end of the ordering
	      elements[0].focus();
	   }
	});
}